# This file is generated by objective.metadata
#
# Last update: Sun Oct 20 21:44:44 2019
#
# flake8: noqa

import sys

import objc

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$kCFErrorDomainCGImageMetadata@^{__CFString=}$kCGImageAnimationDelayTime@^{__CFString=}$kCGImageAnimationLoopCount@^{__CFString=}$kCGImageAnimationStartIndex@^{__CFString=}$kCGImageAuxiliaryDataInfoData@^{__CFString=}$kCGImageAuxiliaryDataInfoDataDescription@^{__CFString=}$kCGImageAuxiliaryDataInfoMetadata@^{__CFString=}$kCGImageAuxiliaryDataTypeDepth@^{__CFString=}$kCGImageAuxiliaryDataTypeDisparity@^{__CFString=}$kCGImageAuxiliaryDataTypePortraitEffectsMatte@^{__CFString=}$kCGImageAuxiliaryDataTypeSemanticSegmentationHairMatte@^{__CFString=}$kCGImageAuxiliaryDataTypeSemanticSegmentationSkinMatte@^{__CFString=}$kCGImageAuxiliaryDataTypeSemanticSegmentationTeethMatte@^{__CFString=}$kCGImageDestinationBackgroundColor@^{__CFString=}$kCGImageDestinationDateTime@^{__CFString=}$kCGImageDestinationEmbedThumbnail@^{__CFString=}$kCGImageDestinationImageMaxPixelSize@^{__CFString=}$kCGImageDestinationLossyCompressionQuality@^{__CFString=}$kCGImageDestinationMergeMetadata@^{__CFString=}$kCGImageDestinationMetadata@^{__CFString=}$kCGImageDestinationOptimizeColorForSharing@^{__CFString=}$kCGImageDestinationOrientation@^{__CFString=}$kCGImageMetadataEnumerateRecursively@^{__CFString=}$kCGImageMetadataNamespaceDublinCore@^{__CFString=}$kCGImageMetadataNamespaceExif@^{__CFString=}$kCGImageMetadataNamespaceExifAux@^{__CFString=}$kCGImageMetadataNamespaceExifEX@^{__CFString=}$kCGImageMetadataNamespaceIPTCCore@^{__CFString=}$kCGImageMetadataNamespaceIPTCExtension@^{__CFString=}$kCGImageMetadataNamespacePhotoshop@^{__CFString=}$kCGImageMetadataNamespaceTIFF@^{__CFString=}$kCGImageMetadataNamespaceXMPBasic@^{__CFString=}$kCGImageMetadataNamespaceXMPRights@^{__CFString=}$kCGImageMetadataPrefixDublinCore@^{__CFString=}$kCGImageMetadataPrefixExif@^{__CFString=}$kCGImageMetadataPrefixExifAux@^{__CFString=}$kCGImageMetadataPrefixExifEX@^{__CFString=}$kCGImageMetadataPrefixIPTCCore@^{__CFString=}$kCGImageMetadataPrefixIPTCExtension@^{__CFString=}$kCGImageMetadataPrefixPhotoshop@^{__CFString=}$kCGImageMetadataPrefixTIFF@^{__CFString=}$kCGImageMetadataPrefixXMPBasic@^{__CFString=}$kCGImageMetadataPrefixXMPRights@^{__CFString=}$kCGImageMetadataShouldExcludeGPS@^{__CFString=}$kCGImageMetadataShouldExcludeXMP@^{__CFString=}$kCGImageProperty8BIMDictionary@^{__CFString=}$kCGImageProperty8BIMLayerNames@^{__CFString=}$kCGImageProperty8BIMVersion@^{__CFString=}$kCGImagePropertyAPNGCanvasPixelHeight@^{__CFString=}$kCGImagePropertyAPNGCanvasPixelWidth@^{__CFString=}$kCGImagePropertyAPNGDelayTime@^{__CFString=}$kCGImagePropertyAPNGFrameInfoArray@^{__CFString=}$kCGImagePropertyAPNGLoopCount@^{__CFString=}$kCGImagePropertyAPNGUnclampedDelayTime@^{__CFString=}$kCGImagePropertyAuxiliaryData@^{__CFString=}$kCGImagePropertyAuxiliaryDataType@^{__CFString=}$kCGImagePropertyBytesPerRow@^{__CFString=}$kCGImagePropertyCIFFCameraSerialNumber@^{__CFString=}$kCGImagePropertyCIFFContinuousDrive@^{__CFString=}$kCGImagePropertyCIFFDescription@^{__CFString=}$kCGImagePropertyCIFFDictionary@^{__CFString=}$kCGImagePropertyCIFFFirmware@^{__CFString=}$kCGImagePropertyCIFFFlashExposureComp@^{__CFString=}$kCGImagePropertyCIFFFocusMode@^{__CFString=}$kCGImagePropertyCIFFImageFileName@^{__CFString=}$kCGImagePropertyCIFFImageName@^{__CFString=}$kCGImagePropertyCIFFImageSerialNumber@^{__CFString=}$kCGImagePropertyCIFFLensMaxMM@^{__CFString=}$kCGImagePropertyCIFFLensMinMM@^{__CFString=}$kCGImagePropertyCIFFLensModel@^{__CFString=}$kCGImagePropertyCIFFMeasuredEV@^{__CFString=}$kCGImagePropertyCIFFMeteringMode@^{__CFString=}$kCGImagePropertyCIFFOwnerName@^{__CFString=}$kCGImagePropertyCIFFRecordID@^{__CFString=}$kCGImagePropertyCIFFReleaseMethod@^{__CFString=}$kCGImagePropertyCIFFReleaseTiming@^{__CFString=}$kCGImagePropertyCIFFSelfTimingTime@^{__CFString=}$kCGImagePropertyCIFFShootingMode@^{__CFString=}$kCGImagePropertyCIFFWhiteBalanceIndex@^{__CFString=}$kCGImagePropertyColorModel@^{__CFString=}$kCGImagePropertyColorModelCMYK@^{__CFString=}$kCGImagePropertyColorModelGray@^{__CFString=}$kCGImagePropertyColorModelLab@^{__CFString=}$kCGImagePropertyColorModelRGB@^{__CFString=}$kCGImagePropertyDNGActiveArea@^{__CFString=}$kCGImagePropertyDNGAnalogBalance@^{__CFString=}$kCGImagePropertyDNGAntiAliasStrength@^{__CFString=}$kCGImagePropertyDNGAsShotICCProfile@^{__CFString=}$kCGImagePropertyDNGAsShotNeutral@^{__CFString=}$kCGImagePropertyDNGAsShotPreProfileMatrix@^{__CFString=}$kCGImagePropertyDNGAsShotProfileName@^{__CFString=}$kCGImagePropertyDNGAsShotWhiteXY@^{__CFString=}$kCGImagePropertyDNGBackwardVersion@^{__CFString=}$kCGImagePropertyDNGBaselineExposure@^{__CFString=}$kCGImagePropertyDNGBaselineExposureOffset@^{__CFString=}$kCGImagePropertyDNGBaselineNoise@^{__CFString=}$kCGImagePropertyDNGBaselineSharpness@^{__CFString=}$kCGImagePropertyDNGBayerGreenSplit@^{__CFString=}$kCGImagePropertyDNGBestQualityScale@^{__CFString=}$kCGImagePropertyDNGBlackLevel@^{__CFString=}$kCGImagePropertyDNGBlackLevelDeltaH@^{__CFString=}$kCGImagePropertyDNGBlackLevelDeltaV@^{__CFString=}$kCGImagePropertyDNGBlackLevelRepeatDim@^{__CFString=}$kCGImagePropertyDNGCFALayout@^{__CFString=}$kCGImagePropertyDNGCFAPlaneColor@^{__CFString=}$kCGImagePropertyDNGCalibrationIlluminant1@^{__CFString=}$kCGImagePropertyDNGCalibrationIlluminant2@^{__CFString=}$kCGImagePropertyDNGCameraCalibration1@^{__CFString=}$kCGImagePropertyDNGCameraCalibration2@^{__CFString=}$kCGImagePropertyDNGCameraCalibrationSignature@^{__CFString=}$kCGImagePropertyDNGCameraSerialNumber@^{__CFString=}$kCGImagePropertyDNGChromaBlurRadius@^{__CFString=}$kCGImagePropertyDNGColorMatrix1@^{__CFString=}$kCGImagePropertyDNGColorMatrix2@^{__CFString=}$kCGImagePropertyDNGColorimetricReference@^{__CFString=}$kCGImagePropertyDNGCurrentICCProfile@^{__CFString=}$kCGImagePropertyDNGCurrentPreProfileMatrix@^{__CFString=}$kCGImagePropertyDNGDefaultBlackRender@^{__CFString=}$kCGImagePropertyDNGDefaultCropOrigin@^{__CFString=}$kCGImagePropertyDNGDefaultCropSize@^{__CFString=}$kCGImagePropertyDNGDefaultScale@^{__CFString=}$kCGImagePropertyDNGDefaultUserCrop@^{__CFString=}$kCGImagePropertyDNGDictionary@^{__CFString=}$kCGImagePropertyDNGExtraCameraProfiles@^{__CFString=}$kCGImagePropertyDNGFixVignetteRadial@^{__CFString=}$kCGImagePropertyDNGForwardMatrix1@^{__CFString=}$kCGImagePropertyDNGForwardMatrix2@^{__CFString=}$kCGImagePropertyDNGLensInfo@^{__CFString=}$kCGImagePropertyDNGLinearResponseLimit@^{__CFString=}$kCGImagePropertyDNGLinearizationTable@^{__CFString=}$kCGImagePropertyDNGLocalizedCameraModel@^{__CFString=}$kCGImagePropertyDNGMakerNoteSafety@^{__CFString=}$kCGImagePropertyDNGMaskedAreas@^{__CFString=}$kCGImagePropertyDNGNewRawImageDigest@^{__CFString=}$kCGImagePropertyDNGNoiseProfile@^{__CFString=}$kCGImagePropertyDNGNoiseReductionApplied@^{__CFString=}$kCGImagePropertyDNGOpcodeList1@^{__CFString=}$kCGImagePropertyDNGOpcodeList2@^{__CFString=}$kCGImagePropertyDNGOpcodeList3@^{__CFString=}$kCGImagePropertyDNGOriginalBestQualityFinalSize@^{__CFString=}$kCGImagePropertyDNGOriginalDefaultCropSize@^{__CFString=}$kCGImagePropertyDNGOriginalDefaultFinalSize@^{__CFString=}$kCGImagePropertyDNGOriginalRawFileData@^{__CFString=}$kCGImagePropertyDNGOriginalRawFileDigest@^{__CFString=}$kCGImagePropertyDNGOriginalRawFileName@^{__CFString=}$kCGImagePropertyDNGPreviewApplicationName@^{__CFString=}$kCGImagePropertyDNGPreviewApplicationVersion@^{__CFString=}$kCGImagePropertyDNGPreviewColorSpace@^{__CFString=}$kCGImagePropertyDNGPreviewDateTime@^{__CFString=}$kCGImagePropertyDNGPreviewSettingsDigest@^{__CFString=}$kCGImagePropertyDNGPreviewSettingsName@^{__CFString=}$kCGImagePropertyDNGPrivateData@^{__CFString=}$kCGImagePropertyDNGProfileCalibrationSignature@^{__CFString=}$kCGImagePropertyDNGProfileCopyright@^{__CFString=}$kCGImagePropertyDNGProfileEmbedPolicy@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapData1@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapData2@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapDims@^{__CFString=}$kCGImagePropertyDNGProfileHueSatMapEncoding@^{__CFString=}$kCGImagePropertyDNGProfileLookTableData@^{__CFString=}$kCGImagePropertyDNGProfileLookTableDims@^{__CFString=}$kCGImagePropertyDNGProfileLookTableEncoding@^{__CFString=}$kCGImagePropertyDNGProfileName@^{__CFString=}$kCGImagePropertyDNGProfileToneCurve@^{__CFString=}$kCGImagePropertyDNGRawDataUniqueID@^{__CFString=}$kCGImagePropertyDNGRawImageDigest@^{__CFString=}$kCGImagePropertyDNGRawToPreviewGain@^{__CFString=}$kCGImagePropertyDNGReductionMatrix1@^{__CFString=}$kCGImagePropertyDNGReductionMatrix2@^{__CFString=}$kCGImagePropertyDNGRowInterleaveFactor@^{__CFString=}$kCGImagePropertyDNGShadowScale@^{__CFString=}$kCGImagePropertyDNGSubTileBlockSize@^{__CFString=}$kCGImagePropertyDNGUniqueCameraModel@^{__CFString=}$kCGImagePropertyDNGVersion@^{__CFString=}$kCGImagePropertyDNGWarpFisheye@^{__CFString=}$kCGImagePropertyDNGWarpRectilinear@^{__CFString=}$kCGImagePropertyDNGWhiteLevel@^{__CFString=}$kCGImagePropertyDPIHeight@^{__CFString=}$kCGImagePropertyDPIWidth@^{__CFString=}$kCGImagePropertyDepth@^{__CFString=}$kCGImagePropertyExifApertureValue@^{__CFString=}$kCGImagePropertyExifAuxDictionary@^{__CFString=}$kCGImagePropertyExifAuxFirmware@^{__CFString=}$kCGImagePropertyExifAuxFlashCompensation@^{__CFString=}$kCGImagePropertyExifAuxImageNumber@^{__CFString=}$kCGImagePropertyExifAuxLensID@^{__CFString=}$kCGImagePropertyExifAuxLensInfo@^{__CFString=}$kCGImagePropertyExifAuxLensModel@^{__CFString=}$kCGImagePropertyExifAuxLensSerialNumber@^{__CFString=}$kCGImagePropertyExifAuxOwnerName@^{__CFString=}$kCGImagePropertyExifAuxSerialNumber@^{__CFString=}$kCGImagePropertyExifBodySerialNumber@^{__CFString=}$kCGImagePropertyExifBrightnessValue@^{__CFString=}$kCGImagePropertyExifCFAPattern@^{__CFString=}$kCGImagePropertyExifCameraOwnerName@^{__CFString=}$kCGImagePropertyExifColorSpace@^{__CFString=}$kCGImagePropertyExifComponentsConfiguration@^{__CFString=}$kCGImagePropertyExifCompositeImage$kCGImagePropertyExifCompressedBitsPerPixel@^{__CFString=}$kCGImagePropertyExifContrast@^{__CFString=}$kCGImagePropertyExifCustomRendered@^{__CFString=}$kCGImagePropertyExifDateTimeDigitized@^{__CFString=}$kCGImagePropertyExifDateTimeOriginal@^{__CFString=}$kCGImagePropertyExifDeviceSettingDescription@^{__CFString=}$kCGImagePropertyExifDictionary@^{__CFString=}$kCGImagePropertyExifDigitalZoomRatio@^{__CFString=}$kCGImagePropertyExifExposureBiasValue@^{__CFString=}$kCGImagePropertyExifExposureIndex@^{__CFString=}$kCGImagePropertyExifExposureMode@^{__CFString=}$kCGImagePropertyExifExposureProgram@^{__CFString=}$kCGImagePropertyExifExposureTime@^{__CFString=}$kCGImagePropertyExifFNumber@^{__CFString=}$kCGImagePropertyExifFileSource@^{__CFString=}$kCGImagePropertyExifFlash@^{__CFString=}$kCGImagePropertyExifFlashEnergy@^{__CFString=}$kCGImagePropertyExifFlashPixVersion@^{__CFString=}$kCGImagePropertyExifFocalLenIn35mmFilm@^{__CFString=}$kCGImagePropertyExifFocalLength@^{__CFString=}$kCGImagePropertyExifFocalPlaneResolutionUnit@^{__CFString=}$kCGImagePropertyExifFocalPlaneXResolution@^{__CFString=}$kCGImagePropertyExifFocalPlaneYResolution@^{__CFString=}$kCGImagePropertyExifGainControl@^{__CFString=}$kCGImagePropertyExifGamma@^{__CFString=}$kCGImagePropertyExifISOSpeed@^{__CFString=}$kCGImagePropertyExifISOSpeedLatitudeyyy@^{__CFString=}$kCGImagePropertyExifISOSpeedLatitudezzz@^{__CFString=}$kCGImagePropertyExifISOSpeedRatings@^{__CFString=}$kCGImagePropertyExifImageUniqueID@^{__CFString=}$kCGImagePropertyExifLensMake@^{__CFString=}$kCGImagePropertyExifLensModel@^{__CFString=}$kCGImagePropertyExifLensSerialNumber@^{__CFString=}$kCGImagePropertyExifLensSpecification@^{__CFString=}$kCGImagePropertyExifLightSource@^{__CFString=}$kCGImagePropertyExifMakerNote@^{__CFString=}$kCGImagePropertyExifMaxApertureValue@^{__CFString=}$kCGImagePropertyExifMeteringMode@^{__CFString=}$kCGImagePropertyExifOECF@^{__CFString=}$kCGImagePropertyExifOffsetTime@^{__CFString=}$kCGImagePropertyExifOffsetTimeDigitized@^{__CFString=}$kCGImagePropertyExifOffsetTimeOriginal@^{__CFString=}$kCGImagePropertyExifPixelXDimension@^{__CFString=}$kCGImagePropertyExifPixelYDimension@^{__CFString=}$kCGImagePropertyExifRecommendedExposureIndex@^{__CFString=}$kCGImagePropertyExifRelatedSoundFile@^{__CFString=}$kCGImagePropertyExifSaturation@^{__CFString=}$kCGImagePropertyExifSceneCaptureType@^{__CFString=}$kCGImagePropertyExifSceneType@^{__CFString=}$kCGImagePropertyExifSensingMethod@^{__CFString=}$kCGImagePropertyExifSensitivityType@^{__CFString=}$kCGImagePropertyExifSharpness@^{__CFString=}$kCGImagePropertyExifShutterSpeedValue@^{__CFString=}$kCGImagePropertyExifSourceExposureTimesOfCompositeImage$kCGImagePropertyExifSourceImageNumberOfCompositeImage$kCGImagePropertyExifSpatialFrequencyResponse@^{__CFString=}$kCGImagePropertyExifSpectralSensitivity@^{__CFString=}$kCGImagePropertyExifStandardOutputSensitivity@^{__CFString=}$kCGImagePropertyExifSubjectArea@^{__CFString=}$kCGImagePropertyExifSubjectDistRange@^{__CFString=}$kCGImagePropertyExifSubjectDistance@^{__CFString=}$kCGImagePropertyExifSubjectLocation@^{__CFString=}$kCGImagePropertyExifSubsecTime@^{__CFString=}$kCGImagePropertyExifSubsecTimeDigitized@^{__CFString=}$kCGImagePropertyExifSubsecTimeOrginal@^{__CFString=}$kCGImagePropertyExifSubsecTimeOriginal@^{__CFString=}$kCGImagePropertyExifUserComment@^{__CFString=}$kCGImagePropertyExifVersion@^{__CFString=}$kCGImagePropertyExifWhiteBalance@^{__CFString=}$kCGImagePropertyFileContentsDictionary@^{__CFString=}$kCGImagePropertyFileSize@^{__CFString=}$kCGImagePropertyGIFCanvasPixelHeight@^{__CFString=}$kCGImagePropertyGIFCanvasPixelWidth@^{__CFString=}$kCGImagePropertyGIFDelayTime@^{__CFString=}$kCGImagePropertyGIFDictionary@^{__CFString=}$kCGImagePropertyGIFFrameInfoArray@^{__CFString=}$kCGImagePropertyGIFHasGlobalColorMap@^{__CFString=}$kCGImagePropertyGIFImageColorMap@^{__CFString=}$kCGImagePropertyGIFLoopCount@^{__CFString=}$kCGImagePropertyGIFUnclampedDelayTime@^{__CFString=}$kCGImagePropertyGPSAltitude@^{__CFString=}$kCGImagePropertyGPSAltitudeRef@^{__CFString=}$kCGImagePropertyGPSAreaInformation@^{__CFString=}$kCGImagePropertyGPSDOP@^{__CFString=}$kCGImagePropertyGPSDateStamp@^{__CFString=}$kCGImagePropertyGPSDestBearing@^{__CFString=}$kCGImagePropertyGPSDestBearingRef@^{__CFString=}$kCGImagePropertyGPSDestDistance@^{__CFString=}$kCGImagePropertyGPSDestDistanceRef@^{__CFString=}$kCGImagePropertyGPSDestLatitude@^{__CFString=}$kCGImagePropertyGPSDestLatitudeRef@^{__CFString=}$kCGImagePropertyGPSDestLongitude@^{__CFString=}$kCGImagePropertyGPSDestLongitudeRef@^{__CFString=}$kCGImagePropertyGPSDictionary@^{__CFString=}$kCGImagePropertyGPSDifferental@^{__CFString=}$kCGImagePropertyGPSHPositioningError@^{__CFString=}$kCGImagePropertyGPSImgDirection@^{__CFString=}$kCGImagePropertyGPSImgDirectionRef@^{__CFString=}$kCGImagePropertyGPSLatitude@^{__CFString=}$kCGImagePropertyGPSLatitudeRef@^{__CFString=}$kCGImagePropertyGPSLongitude@^{__CFString=}$kCGImagePropertyGPSLongitudeRef@^{__CFString=}$kCGImagePropertyGPSMapDatum@^{__CFString=}$kCGImagePropertyGPSMeasureMode@^{__CFString=}$kCGImagePropertyGPSProcessingMethod@^{__CFString=}$kCGImagePropertyGPSSatellites@^{__CFString=}$kCGImagePropertyGPSSpeed@^{__CFString=}$kCGImagePropertyGPSSpeedRef@^{__CFString=}$kCGImagePropertyGPSStatus@^{__CFString=}$kCGImagePropertyGPSTimeStamp@^{__CFString=}$kCGImagePropertyGPSTrack@^{__CFString=}$kCGImagePropertyGPSTrackRef@^{__CFString=}$kCGImagePropertyGPSVersion@^{__CFString=}$kCGImagePropertyHEICSCanvasPixelHeight@^{__CFString=}$kCGImagePropertyHEICSCanvasPixelWidth@^{__CFString=}$kCGImagePropertyHEICSDelayTime@^{__CFString=}$kCGImagePropertyHEICSDictionary@^{__CFString=}$kCGImagePropertyHEICSFrameInfoArray@^{__CFString=}$kCGImagePropertyHEICSLoopCount@^{__CFString=}$kCGImagePropertyHEICSUnclampedDelayTime@^{__CFString=}$kCGImagePropertyHasAlpha@^{__CFString=}$kCGImagePropertyHeight@^{__CFString=}$kCGImagePropertyIPTCActionAdvised@^{__CFString=}$kCGImagePropertyIPTCByline@^{__CFString=}$kCGImagePropertyIPTCBylineTitle@^{__CFString=}$kCGImagePropertyIPTCCaptionAbstract@^{__CFString=}$kCGImagePropertyIPTCCategory@^{__CFString=}$kCGImagePropertyIPTCCity@^{__CFString=}$kCGImagePropertyIPTCContact@^{__CFString=}$kCGImagePropertyIPTCContactInfoAddress@^{__CFString=}$kCGImagePropertyIPTCContactInfoCity@^{__CFString=}$kCGImagePropertyIPTCContactInfoCountry@^{__CFString=}$kCGImagePropertyIPTCContactInfoEmails@^{__CFString=}$kCGImagePropertyIPTCContactInfoPhones@^{__CFString=}$kCGImagePropertyIPTCContactInfoPostalCode@^{__CFString=}$kCGImagePropertyIPTCContactInfoStateProvince@^{__CFString=}$kCGImagePropertyIPTCContactInfoWebURLs@^{__CFString=}$kCGImagePropertyIPTCContentLocationCode@^{__CFString=}$kCGImagePropertyIPTCContentLocationName@^{__CFString=}$kCGImagePropertyIPTCCopyrightNotice@^{__CFString=}$kCGImagePropertyIPTCCountryPrimaryLocationCode@^{__CFString=}$kCGImagePropertyIPTCCountryPrimaryLocationName@^{__CFString=}$kCGImagePropertyIPTCCreatorContactInfo@^{__CFString=}$kCGImagePropertyIPTCCredit@^{__CFString=}$kCGImagePropertyIPTCDateCreated@^{__CFString=}$kCGImagePropertyIPTCDictionary@^{__CFString=}$kCGImagePropertyIPTCDigitalCreationDate@^{__CFString=}$kCGImagePropertyIPTCDigitalCreationTime@^{__CFString=}$kCGImagePropertyIPTCEditStatus@^{__CFString=}$kCGImagePropertyIPTCEditorialUpdate@^{__CFString=}$kCGImagePropertyIPTCExpirationDate@^{__CFString=}$kCGImagePropertyIPTCExpirationTime@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTerm@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermCvId@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtAboutCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCExtAddlModelInfo@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCircaDateCreated@^{__CFString=}$kCGImagePropertyIPTCExtArtworkContentDescription@^{__CFString=}$kCGImagePropertyIPTCExtArtworkContributionDescription@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCopyrightNotice@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCopyrightOwnerID@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCopyrightOwnerName@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCreator@^{__CFString=}$kCGImagePropertyIPTCExtArtworkCreatorID@^{__CFString=}$kCGImagePropertyIPTCExtArtworkDateCreated@^{__CFString=}$kCGImagePropertyIPTCExtArtworkLicensorID@^{__CFString=}$kCGImagePropertyIPTCExtArtworkLicensorName@^{__CFString=}$kCGImagePropertyIPTCExtArtworkOrObject@^{__CFString=}$kCGImagePropertyIPTCExtArtworkPhysicalDescription@^{__CFString=}$kCGImagePropertyIPTCExtArtworkSource@^{__CFString=}$kCGImagePropertyIPTCExtArtworkSourceInvURL@^{__CFString=}$kCGImagePropertyIPTCExtArtworkSourceInventoryNo@^{__CFString=}$kCGImagePropertyIPTCExtArtworkStylePeriod@^{__CFString=}$kCGImagePropertyIPTCExtArtworkTitle@^{__CFString=}$kCGImagePropertyIPTCExtAudioBitrate@^{__CFString=}$kCGImagePropertyIPTCExtAudioBitrateMode@^{__CFString=}$kCGImagePropertyIPTCExtAudioChannelCount@^{__CFString=}$kCGImagePropertyIPTCExtCircaDateCreated@^{__CFString=}$kCGImagePropertyIPTCExtContainerFormat@^{__CFString=}$kCGImagePropertyIPTCExtContainerFormatIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtContainerFormatName@^{__CFString=}$kCGImagePropertyIPTCExtContributor@^{__CFString=}$kCGImagePropertyIPTCExtContributorIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtContributorName@^{__CFString=}$kCGImagePropertyIPTCExtContributorRole@^{__CFString=}$kCGImagePropertyIPTCExtControlledVocabularyTerm@^{__CFString=}$kCGImagePropertyIPTCExtCopyrightYear@^{__CFString=}$kCGImagePropertyIPTCExtCreator@^{__CFString=}$kCGImagePropertyIPTCExtCreatorIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtCreatorName@^{__CFString=}$kCGImagePropertyIPTCExtCreatorRole@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreen@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegion@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionD@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionH@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionText@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionUnit@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionW@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionX@^{__CFString=}$kCGImagePropertyIPTCExtDataOnScreenRegionY@^{__CFString=}$kCGImagePropertyIPTCExtDigitalImageGUID@^{__CFString=}$kCGImagePropertyIPTCExtDigitalSourceFileType@^{__CFString=}$kCGImagePropertyIPTCExtDigitalSourceType@^{__CFString=}$kCGImagePropertyIPTCExtDopesheet@^{__CFString=}$kCGImagePropertyIPTCExtDopesheetLink@^{__CFString=}$kCGImagePropertyIPTCExtDopesheetLinkLink@^{__CFString=}$kCGImagePropertyIPTCExtDopesheetLinkLinkQualifier@^{__CFString=}$kCGImagePropertyIPTCExtEmbdEncRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtEmbeddedEncodedRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtEmbeddedEncodedRightsExprLangID@^{__CFString=}$kCGImagePropertyIPTCExtEmbeddedEncodedRightsExprType@^{__CFString=}$kCGImagePropertyIPTCExtEpisode@^{__CFString=}$kCGImagePropertyIPTCExtEpisodeIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtEpisodeName@^{__CFString=}$kCGImagePropertyIPTCExtEpisodeNumber@^{__CFString=}$kCGImagePropertyIPTCExtEvent@^{__CFString=}$kCGImagePropertyIPTCExtExternalMetadataLink@^{__CFString=}$kCGImagePropertyIPTCExtFeedIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtGenre@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvId@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtGenreCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCExtHeadline@^{__CFString=}$kCGImagePropertyIPTCExtIPTCLastEdited@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncodedRightsExpr@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncodedRightsExprLangID@^{__CFString=}$kCGImagePropertyIPTCExtLinkedEncodedRightsExprType@^{__CFString=}$kCGImagePropertyIPTCExtLocationCity@^{__CFString=}$kCGImagePropertyIPTCExtLocationCountryCode@^{__CFString=}$kCGImagePropertyIPTCExtLocationCountryName@^{__CFString=}$kCGImagePropertyIPTCExtLocationCreated@^{__CFString=}$kCGImagePropertyIPTCExtLocationGPSAltitude@^{__CFString=}$kCGImagePropertyIPTCExtLocationGPSLatitude@^{__CFString=}$kCGImagePropertyIPTCExtLocationGPSLongitude@^{__CFString=}$kCGImagePropertyIPTCExtLocationIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtLocationLocationId@^{__CFString=}$kCGImagePropertyIPTCExtLocationLocationName@^{__CFString=}$kCGImagePropertyIPTCExtLocationProvinceState@^{__CFString=}$kCGImagePropertyIPTCExtLocationShown@^{__CFString=}$kCGImagePropertyIPTCExtLocationSublocation@^{__CFString=}$kCGImagePropertyIPTCExtLocationWorldRegion@^{__CFString=}$kCGImagePropertyIPTCExtMaxAvailHeight@^{__CFString=}$kCGImagePropertyIPTCExtMaxAvailWidth@^{__CFString=}$kCGImagePropertyIPTCExtModelAge@^{__CFString=}$kCGImagePropertyIPTCExtOrganisationInImageCode@^{__CFString=}$kCGImagePropertyIPTCExtOrganisationInImageName@^{__CFString=}$kCGImagePropertyIPTCExtPersonHeard@^{__CFString=}$kCGImagePropertyIPTCExtPersonHeardIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtPersonHeardName@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImage@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCharacteristic@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermCvId@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageDescription@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageId@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageName@^{__CFString=}$kCGImagePropertyIPTCExtPersonInImageWDetails@^{__CFString=}$kCGImagePropertyIPTCExtProductInImage@^{__CFString=}$kCGImagePropertyIPTCExtProductInImageDescription@^{__CFString=}$kCGImagePropertyIPTCExtProductInImageGTIN@^{__CFString=}$kCGImagePropertyIPTCExtProductInImageName@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEvent@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEventDate@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEventIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtPublicationEventName@^{__CFString=}$kCGImagePropertyIPTCExtRating@^{__CFString=}$kCGImagePropertyIPTCExtRatingRatingRegion@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionCity@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionCountryCode@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionCountryName@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionGPSAltitude@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionGPSLatitude@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionGPSLongitude@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionLocationId@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionLocationName@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionProvinceState@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionSublocation@^{__CFString=}$kCGImagePropertyIPTCExtRatingRegionWorldRegion@^{__CFString=}$kCGImagePropertyIPTCExtRatingScaleMaxValue@^{__CFString=}$kCGImagePropertyIPTCExtRatingScaleMinValue@^{__CFString=}$kCGImagePropertyIPTCExtRatingSourceLink@^{__CFString=}$kCGImagePropertyIPTCExtRatingValue@^{__CFString=}$kCGImagePropertyIPTCExtRatingValueLogoLink@^{__CFString=}$kCGImagePropertyIPTCExtRegistryEntryRole@^{__CFString=}$kCGImagePropertyIPTCExtRegistryID@^{__CFString=}$kCGImagePropertyIPTCExtRegistryItemID@^{__CFString=}$kCGImagePropertyIPTCExtRegistryOrganisationID@^{__CFString=}$kCGImagePropertyIPTCExtReleaseReady@^{__CFString=}$kCGImagePropertyIPTCExtSeason@^{__CFString=}$kCGImagePropertyIPTCExtSeasonIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtSeasonName@^{__CFString=}$kCGImagePropertyIPTCExtSeasonNumber@^{__CFString=}$kCGImagePropertyIPTCExtSeries@^{__CFString=}$kCGImagePropertyIPTCExtSeriesIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtSeriesName@^{__CFString=}$kCGImagePropertyIPTCExtShownEvent@^{__CFString=}$kCGImagePropertyIPTCExtShownEventIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtShownEventName@^{__CFString=}$kCGImagePropertyIPTCExtStorylineIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtStreamReady@^{__CFString=}$kCGImagePropertyIPTCExtStylePeriod@^{__CFString=}$kCGImagePropertyIPTCExtSupplyChainSource@^{__CFString=}$kCGImagePropertyIPTCExtSupplyChainSourceIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtSupplyChainSourceName@^{__CFString=}$kCGImagePropertyIPTCExtTemporalCoverage@^{__CFString=}$kCGImagePropertyIPTCExtTemporalCoverageFrom@^{__CFString=}$kCGImagePropertyIPTCExtTemporalCoverageTo@^{__CFString=}$kCGImagePropertyIPTCExtTranscript@^{__CFString=}$kCGImagePropertyIPTCExtTranscriptLink@^{__CFString=}$kCGImagePropertyIPTCExtTranscriptLinkLink@^{__CFString=}$kCGImagePropertyIPTCExtTranscriptLinkLinkQualifier@^{__CFString=}$kCGImagePropertyIPTCExtVideoBitrate@^{__CFString=}$kCGImagePropertyIPTCExtVideoBitrateMode@^{__CFString=}$kCGImagePropertyIPTCExtVideoDisplayAspectRatio@^{__CFString=}$kCGImagePropertyIPTCExtVideoEncodingProfile@^{__CFString=}$kCGImagePropertyIPTCExtVideoShotType@^{__CFString=}$kCGImagePropertyIPTCExtVideoShotTypeIdentifier@^{__CFString=}$kCGImagePropertyIPTCExtVideoShotTypeName@^{__CFString=}$kCGImagePropertyIPTCExtVideoStreamsCount@^{__CFString=}$kCGImagePropertyIPTCExtVisualColor@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTag@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvId@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvTermId@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvTermName@^{__CFString=}$kCGImagePropertyIPTCExtWorkflowTagCvTermRefinedAbout@^{__CFString=}$kCGImagePropertyIPTCFixtureIdentifier@^{__CFString=}$kCGImagePropertyIPTCHeadline@^{__CFString=}$kCGImagePropertyIPTCImageOrientation@^{__CFString=}$kCGImagePropertyIPTCImageType@^{__CFString=}$kCGImagePropertyIPTCKeywords@^{__CFString=}$kCGImagePropertyIPTCLanguageIdentifier@^{__CFString=}$kCGImagePropertyIPTCObjectAttributeReference@^{__CFString=}$kCGImagePropertyIPTCObjectCycle@^{__CFString=}$kCGImagePropertyIPTCObjectName@^{__CFString=}$kCGImagePropertyIPTCObjectTypeReference@^{__CFString=}$kCGImagePropertyIPTCOriginalTransmissionReference@^{__CFString=}$kCGImagePropertyIPTCOriginatingProgram@^{__CFString=}$kCGImagePropertyIPTCProgramVersion@^{__CFString=}$kCGImagePropertyIPTCProvinceState@^{__CFString=}$kCGImagePropertyIPTCReferenceDate@^{__CFString=}$kCGImagePropertyIPTCReferenceNumber@^{__CFString=}$kCGImagePropertyIPTCReferenceService@^{__CFString=}$kCGImagePropertyIPTCReleaseDate@^{__CFString=}$kCGImagePropertyIPTCReleaseTime@^{__CFString=}$kCGImagePropertyIPTCRightsUsageTerms@^{__CFString=}$kCGImagePropertyIPTCScene@^{__CFString=}$kCGImagePropertyIPTCSource@^{__CFString=}$kCGImagePropertyIPTCSpecialInstructions@^{__CFString=}$kCGImagePropertyIPTCStarRating@^{__CFString=}$kCGImagePropertyIPTCSubLocation@^{__CFString=}$kCGImagePropertyIPTCSubjectReference@^{__CFString=}$kCGImagePropertyIPTCSupplementalCategory@^{__CFString=}$kCGImagePropertyIPTCTimeCreated@^{__CFString=}$kCGImagePropertyIPTCUrgency@^{__CFString=}$kCGImagePropertyIPTCWriterEditor@^{__CFString=}$kCGImagePropertyImageCount@^{__CFString=}$kCGImagePropertyImages@^{__CFString=}$kCGImagePropertyIsFloat@^{__CFString=}$kCGImagePropertyIsIndexed@^{__CFString=}$kCGImagePropertyIsSticker$kCGImagePropertyJFIFDensityUnit@^{__CFString=}$kCGImagePropertyJFIFDictionary@^{__CFString=}$kCGImagePropertyJFIFIsProgressive@^{__CFString=}$kCGImagePropertyJFIFVersion@^{__CFString=}$kCGImagePropertyJFIFXDensity@^{__CFString=}$kCGImagePropertyJFIFYDensity@^{__CFString=}$kCGImagePropertyMakerAppleDictionary@^{__CFString=}$kCGImagePropertyMakerCanonAspectRatioInfo@^{__CFString=}$kCGImagePropertyMakerCanonCameraSerialNumber@^{__CFString=}$kCGImagePropertyMakerCanonContinuousDrive@^{__CFString=}$kCGImagePropertyMakerCanonDictionary@^{__CFString=}$kCGImagePropertyMakerCanonFirmware@^{__CFString=}$kCGImagePropertyMakerCanonFlashExposureComp@^{__CFString=}$kCGImagePropertyMakerCanonImageSerialNumber@^{__CFString=}$kCGImagePropertyMakerCanonLensModel@^{__CFString=}$kCGImagePropertyMakerCanonOwnerName@^{__CFString=}$kCGImagePropertyMakerFujiDictionary@^{__CFString=}$kCGImagePropertyMakerMinoltaDictionary@^{__CFString=}$kCGImagePropertyMakerNikonCameraSerialNumber@^{__CFString=}$kCGImagePropertyMakerNikonColorMode@^{__CFString=}$kCGImagePropertyMakerNikonDictionary@^{__CFString=}$kCGImagePropertyMakerNikonDigitalZoom@^{__CFString=}$kCGImagePropertyMakerNikonFlashExposureComp@^{__CFString=}$kCGImagePropertyMakerNikonFlashSetting@^{__CFString=}$kCGImagePropertyMakerNikonFocusDistance@^{__CFString=}$kCGImagePropertyMakerNikonFocusMode@^{__CFString=}$kCGImagePropertyMakerNikonISOSelection@^{__CFString=}$kCGImagePropertyMakerNikonISOSetting@^{__CFString=}$kCGImagePropertyMakerNikonImageAdjustment@^{__CFString=}$kCGImagePropertyMakerNikonLensAdapter@^{__CFString=}$kCGImagePropertyMakerNikonLensInfo@^{__CFString=}$kCGImagePropertyMakerNikonLensType@^{__CFString=}$kCGImagePropertyMakerNikonQuality@^{__CFString=}$kCGImagePropertyMakerNikonSharpenMode@^{__CFString=}$kCGImagePropertyMakerNikonShootingMode@^{__CFString=}$kCGImagePropertyMakerNikonShutterCount@^{__CFString=}$kCGImagePropertyMakerNikonWhiteBalanceMode@^{__CFString=}$kCGImagePropertyMakerOlympusDictionary@^{__CFString=}$kCGImagePropertyMakerPentaxDictionary@^{__CFString=}$kCGImagePropertyNamedColorSpace@^{__CFString=}$kCGImagePropertyOpenEXRAspectRatio@^{__CFString=}$kCGImagePropertyOpenEXRDictionary@^{__CFString=}$kCGImagePropertyOrientation@^{__CFString=}$kCGImagePropertyPNGAuthor@^{__CFString=}$kCGImagePropertyPNGChromaticities@^{__CFString=}$kCGImagePropertyPNGComment@^{__CFString=}$kCGImagePropertyPNGCompressionFilter@^{__CFString=}$kCGImagePropertyPNGCopyright@^{__CFString=}$kCGImagePropertyPNGCreationTime@^{__CFString=}$kCGImagePropertyPNGDescription@^{__CFString=}$kCGImagePropertyPNGDictionary@^{__CFString=}$kCGImagePropertyPNGDisclaimer@^{__CFString=}$kCGImagePropertyPNGGamma@^{__CFString=}$kCGImagePropertyPNGInterlaceType@^{__CFString=}$kCGImagePropertyPNGModificationTime@^{__CFString=}$kCGImagePropertyPNGSoftware@^{__CFString=}$kCGImagePropertyPNGSource@^{__CFString=}$kCGImagePropertyPNGTitle@^{__CFString=}$kCGImagePropertyPNGWarning@^{__CFString=}$kCGImagePropertyPNGXPixelsPerMeter@^{__CFString=}$kCGImagePropertyPNGYPixelsPerMeter@^{__CFString=}$kCGImagePropertyPNGsRGBIntent@^{__CFString=}$kCGImagePropertyPixelFormat@^{__CFString=}$kCGImagePropertyPixelHeight@^{__CFString=}$kCGImagePropertyPixelWidth@^{__CFString=}$kCGImagePropertyPrimaryImage@^{__CFString=}$kCGImagePropertyProfileName@^{__CFString=}$kCGImagePropertyRawDictionary@^{__CFString=}$kCGImagePropertyTIFFArtist@^{__CFString=}$kCGImagePropertyTIFFCompression@^{__CFString=}$kCGImagePropertyTIFFCopyright@^{__CFString=}$kCGImagePropertyTIFFDateTime@^{__CFString=}$kCGImagePropertyTIFFDictionary@^{__CFString=}$kCGImagePropertyTIFFDocumentName@^{__CFString=}$kCGImagePropertyTIFFHostComputer@^{__CFString=}$kCGImagePropertyTIFFImageDescription@^{__CFString=}$kCGImagePropertyTIFFMake@^{__CFString=}$kCGImagePropertyTIFFModel@^{__CFString=}$kCGImagePropertyTIFFOrientation@^{__CFString=}$kCGImagePropertyTIFFPhotometricInterpretation@^{__CFString=}$kCGImagePropertyTIFFPrimaryChromaticities@^{__CFString=}$kCGImagePropertyTIFFResolutionUnit@^{__CFString=}$kCGImagePropertyTIFFSoftware@^{__CFString=}$kCGImagePropertyTIFFTileLength@^{__CFString=}$kCGImagePropertyTIFFTileWidth@^{__CFString=}$kCGImagePropertyTIFFTransferFunction@^{__CFString=}$kCGImagePropertyTIFFWhitePoint@^{__CFString=}$kCGImagePropertyTIFFXResolution@^{__CFString=}$kCGImagePropertyTIFFYResolution@^{__CFString=}$kCGImagePropertyThumbnailImages@^{__CFString=}$kCGImagePropertyWidth@^{__CFString=}$kCGImageSourceCreateThumbnailFromImageAlways@^{__CFString=}$kCGImageSourceCreateThumbnailFromImageIfAbsent@^{__CFString=}$kCGImageSourceCreateThumbnailWithTransform@^{__CFString=}$kCGImageSourceShouldAllowFloat@^{__CFString=}$kCGImageSourceShouldCache@^{__CFString=}$kCGImageSourceShouldCacheImmediately@^{__CFString=}$kCGImageSourceSubsampleFactor@^{__CFString=}$kCGImageSourceThumbnailMaxPixelSize@^{__CFString=}$kCGImageSourceTypeIdentifierHint@^{__CFString=}$"""
enums = """$IIO_HAS_IOSURFACE@1$IMAGEIO_PNG_ALL_FILTERS@248$IMAGEIO_PNG_FILTER_AVG@64$IMAGEIO_PNG_FILTER_NONE@8$IMAGEIO_PNG_FILTER_PAETH@128$IMAGEIO_PNG_FILTER_SUB@16$IMAGEIO_PNG_FILTER_UP@32$IMAGEIO_PNG_NO_FILTERS@0$kCGImageAnimationStatus_AllocationFailure@-22144$kCGImageAnimationStatus_CorruptInputImage@-22141$kCGImageAnimationStatus_IncompleteInputImage@-22143$kCGImageAnimationStatus_ParameterError@-22140$kCGImageAnimationStatus_UnsupportedFormat@-22142$kCGImageMetadataErrorBadArgument@2$kCGImageMetadataErrorConflictingArguments@3$kCGImageMetadataErrorPrefixConflict@4$kCGImageMetadataErrorUnknown@0$kCGImageMetadataErrorUnsupportedFormat@1$kCGImageMetadataTypeAlternateArray@4$kCGImageMetadataTypeAlternateText@5$kCGImageMetadataTypeArrayOrdered@3$kCGImageMetadataTypeArrayUnordered@2$kCGImageMetadataTypeDefault@0$kCGImageMetadataTypeInvalid@-1$kCGImageMetadataTypeString@1$kCGImageMetadataTypeStructure@6$kCGImagePropertyOrientationDown@3$kCGImagePropertyOrientationDownMirrored@4$kCGImagePropertyOrientationLeft@8$kCGImagePropertyOrientationLeftMirrored@5$kCGImagePropertyOrientationRight@6$kCGImagePropertyOrientationRightMirrored@7$kCGImagePropertyOrientationUp@1$kCGImagePropertyOrientationUpMirrored@2$kCGImageStatusComplete@0$kCGImageStatusIncomplete@-1$kCGImageStatusInvalidData@-4$kCGImageStatusReadingHeader@-2$kCGImageStatusUnexpectedEOF@-5$kCGImageStatusUnknownType@-3$"""
misc.update({})
functions = {
    "CGImageMetadataSetValueMatchingImageProperty": (
        b"B^{CGImageMetadata=}^{__CFString=}^{__CFString=}@",
    ),
    "CGImageMetadataCreateFromXMPData": (
        b"^{CGImageMetadata=}^{__CFData=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceGetCount": (sel32or64(b"L^{CGImageSource=}", b"Q^{CGImageSource=}"),),
    "CGImageSourceCreateWithData": (
        b"^{CGImageSource=}^{__CFData=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyTypeIdentifiers": (
        b"^{__CFArray=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCopyTagWithPath": (
        b"^{CGImageMetadataTag=}^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageDestinationAddAuxiliaryDataInfo": (
        b"v^{CGImageDestination=}^{__CFString=}^{__CFDictionary=}",
    ),
    "CGImageMetadataTagGetType": (b"i^{CGImageMetadataTag=}",),
    "CGImageSourceGetStatus": (b"i^{CGImageSource=}",),
    "CGImageMetadataCreateXMPData": (
        b"^{__CFData=}^{CGImageMetadata=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCreateImageAtIndex": (
        sel32or64(
            b"^{CGImage=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{CGImage=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyNamespace": (
        b"^{__CFString=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyMetadataAtIndex": (
        sel32or64(
            b"^{CGImageMetadata=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{CGImageMetadata=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCreateWithURL": (
        sel32or64(
            b"^{CGImageDestination=}^{__CFURL=}^{__CFString=}L^{__CFDictionary=}",
            b"^{CGImageDestination=}^{__CFURL=}^{__CFString=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceRemoveCacheAtIndex": (
        sel32or64(b"v^{CGImageSource=}L", b"v^{CGImageSource=}Q"),
    ),
    "CGImageSourceCreateThumbnailAtIndex": (
        sel32or64(
            b"^{CGImage=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{CGImage=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCreateIncremental": (
        b"^{CGImageSource=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyPropertiesAtIndex": (
        sel32or64(
            b"^{__CFDictionary=}^{CGImageSource=}L^{__CFDictionary=}",
            b"^{__CFDictionary=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceUpdateData": (b"v^{CGImageSource=}^{__CFData=}B",),
    "CGImageDestinationSetProperties": (b"v^{CGImageDestination=}^{__CFDictionary=}",),
    "CGImageMetadataSetTagWithPath": (
        b"B^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}^{CGImageMetadataTag=}",
    ),
    "CGAnimateImageAtURLWithBlock": (
        b"i^{__CFURL=}^{__CFDictionary=}@?",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "l"},
                            2: {"type": "@"},
                            3: {"type": "o^B"},
                        },
                    }
                }
            }
        },
    ),
    "CGImageDestinationCreateWithDataConsumer": (
        sel32or64(
            b"^{CGImageDestination=}^{CGDataConsumer=}^{__CFString=}L^{__CFDictionary=}",
            b"^{CGImageDestination=}^{CGDataConsumer=}^{__CFString=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCopyTypeIdentifiers": (
        b"^{__CFArray=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCreateWithData": (
        sel32or64(
            b"^{CGImageDestination=}^{__CFData=}^{__CFString=}L^{__CFDictionary=}",
            b"^{CGImageDestination=}^{__CFData=}^{__CFString=}Q^{__CFDictionary=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCopyAuxiliaryDataInfoAtIndex": (
        b"^{__CFDictionary=}^{CGImageSource=}Q^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGAnimateImageDataWithBlock": (
        b"i^{__CFData=}^{__CFDictionary=}@?",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "l"},
                            2: {"type": "@"},
                            3: {"type": "o^B"},
                        },
                    }
                }
            }
        },
    ),
    "CGImageSourceGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageDestinationAddImageAndMetadata": (
        b"v^{CGImageDestination=}^{CGImage=}^{CGImageMetadata=}^{__CFDictionary=}",
    ),
    "CGImageMetadataTagCreate": (
        b"^{CGImageMetadataTag=}^{__CFString=}^{__CFString=}^{__CFString=}i@",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageDestinationCopyImageSource": (
        b"B^{CGImageDestination=}^{CGImageSource=}^{__CFDictionary=}^^{__CFError=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                3: {
                    "already_cfretained": True,
                    "type_modifier": "o",
                    "null_accepted": True,
                }
            },
        },
    ),
    "CGImageSourceUpdateDataProvider": (b"v^{CGImageSource=}^{CGDataProvider=}B",),
    "CGImageDestinationAddImage": (
        b"v^{CGImageDestination=}^{CGImage=}^{__CFDictionary=}",
    ),
    "CGImageMetadataRemoveTagWithPath": (
        b"B^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}",
    ),
    "CGImageSourceGetStatusAtIndex": (
        sel32or64(b"i^{CGImageSource=}L", b"i^{CGImageSource=}Q"),
    ),
    "CGImageSourceCreateWithURL": (
        b"^{CGImageSource=}^{__CFURL=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyValue": (
        b"@^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCopyTags": (
        b"^{__CFArray=}^{CGImageMetadata=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCopyStringValueWithPath": (
        b"^{__CFString=}^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceGetType": (b"^{__CFString=}^{CGImageSource=}",),
    "CGImageDestinationAddImageFromSource": (
        sel32or64(
            b"v^{CGImageDestination=}^{CGImageSource=}L^{__CFDictionary=}",
            b"v^{CGImageDestination=}^{CGImageSource=}Q^{__CFDictionary=}",
        ),
    ),
    "CGImageDestinationGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageDestinationFinalize": (b"B^{CGImageDestination=}",),
    "CGImageSourceGetPrimaryImageIndex": (b"Q^{CGImageSource=}",),
    "CGImageSourceCopyProperties": (
        b"^{__CFDictionary=}^{CGImageSource=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagGetTypeID": (sel32or64(b"L", b"Q"),),
    "CGImageMetadataSetValueWithPath": (
        b"B^{CGImageMetadata=}^{CGImageMetadataTag=}^{__CFString=}@",
    ),
    "CGImageMetadataCopyTagMatchingImageProperty": (
        b"^{CGImageMetadataTag=}^{CGImageMetadata=}^{__CFString=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataRegisterNamespaceForPrefix": (
        b"B^{CGImageMetadata=}^{__CFString=}^{__CFString=}^^{__CFError=}",
        "",
        {
            "arguments": {
                3: {
                    "already_cfretained": True,
                    "type_modifier": "o",
                    "null_accepted": True,
                }
            }
        },
    ),
    "CGImageMetadataTagCopyPrefix": (
        b"^{__CFString=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyName": (
        b"^{__CFString=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataEnumerateTagsUsingBlock": (
        b"v^{CGImageMetadata=}^{__CFString=}^{__CFDictionary=}@?",
        "",
        {
            "arguments": {
                3: {
                    "callable": {
                        "retval": {"type": b"B"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "@"},
                            2: {"type": "@"},
                        },
                    }
                }
            }
        },
    ),
    "CGImageMetadataCreateMutable": (
        b"^{CGImageMetadata=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageSourceCreateWithDataProvider": (
        b"^{CGImageSource=}^{CGDataProvider=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataTagCopyQualifiers": (
        b"^{__CFArray=}^{CGImageMetadataTag=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CGImageMetadataCreateMutableCopy": (
        b"^{CGImageMetadata=}^{CGImageMetadata=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
}
aliases = {
    "IMAGEIO_AVAILABLE_STARTING": "__OSX_AVAILABLE_STARTING",
    "IMAGEIO_AVAILABLE_BUT_DEPRECATED": "__OSX_AVAILABLE_BUT_DEPRECATED",
    "_iio_Nullable": "_Nullable",
    "_iio_Nonnull": "_Nonnull",
}
cftypes = [
    ("CGImageSourceRef", b"^{CGImageSource=}", "CGImageSourceGetTypeID", None),
    (
        "CGImageDestinationRef",
        b"^{CGImageDestination=}",
        "CGImageDestinationGetTypeID",
        None,
    ),
    (
        "CGImageMetadataTagRef",
        b"^{CGImageMetadataTag=}",
        "CGImageMetadataTagGetTypeID",
        None,
    ),
    ("CGImageMetadataRef", b"^{CGImageMetadata=}", "CGImageMetadataGetTypeID", None),
]
expressions = {}

# END OF FILE



"""
Helper module that makes it easier to import all of Quartz
"""
import sys

import AppKit
import objc


def _load():
    submods = []
    sys.modules["Quartz"] = mod = objc.ObjCLazyModule(
        "Quartz",
        None,
        None,
        {},
        None,
        {
            "__doc__": __doc__,
            "objc": objc,
            "__path__": __path__,
            "__loader__": globals().get("__loader__", None),
        },
        submods,
    )

    try:
        from Quartz import CoreGraphics as m

        submods.append(m)
        mod.CoreGraphics = m
    except ImportError:
        pass

    try:
        from Quartz import ImageIO as m

        submods.append(m)
        mod.ImageIO = m
    except ImportError:
        pass

    try:
        from Quartz import ImageKit as m

        submods.append(m)
        mod.ImageIO = m
    except ImportError:
        pass

    try:
        from Quartz import CoreVideo as m

        submods.append(m)
        mod.CoreVideo = m
    except ImportError:
        pass

    try:
        from Quartz import QuartzCore as m

        submods.append(m)
        mod.QuartCore = m
    except ImportError:
        pass

    try:
        from Quartz import ImageIO as m

        submods.append(m)
        mod.ImageIO = m
    except ImportError:
        pass

    try:
        from Quartz import PDFKit as m

        submods.append(m)
        mod.PDFKit = m
    except ImportError:
        pass

    try:
        from Quartz import QuartzFilters as m

        submods.append(m)
        mod.QuartzFilters = m
    except ImportError:
        pass

    try:
        from Quartz import QuickLookUI as m

        submods.append(m)
        mod.QuickLookUI = m
    except ImportError:
        pass

    submods.append(AppKit)


_load()



import os

import Cocoa
import LaunchServices
import objc
import Quartz


#       openFiles
#
#       A simple C function that opens NSOpenPanel and returns an array of file paths.
#       It uses uniform type identifiers (UTIs) for proper filtering of image files.
# -------------------------------------------------------------------------
def openFiles():
    # Get a list of extensions to filter in our NSOpenPanel.
    panel = Cocoa.NSOpenPanel.openPanel()

    # The user can choose a folder; images in the folder are added recursively.
    panel.setCanChooseDirectories_(True)
    panel.setCanChooseFiles_(True)
    panel.setAllowsMultipleSelection_(True)

    if (
        panel.runModalForTypes_(Cocoa.NSImage.imageUnfilteredTypes())
        == Cocoa.NSOKButton
    ):
        return panel.filenames()

    return []


# ==============================================================================
# This is the data source object.
class myImageObject(Cocoa.NSObject):
    _path = objc.ivar()

    # -------------------------------------------------------------------------
    #   setPath:path
    #
    #   The data source object is just a file path representation
    # -------------------------------------------------------------------------
    def setPath_(self, inPath):
        self._path = inPath

    # The required methods of the IKImageBrowserItem protocol.

    # -------------------------------------------------------------------------
    #   imageRepresentationType:
    #
    #   Set up the image browser to use a path representation.
    # -------------------------------------------------------------------------
    def imageRepresentationType(self):
        return Quartz.IKImageBrowserPathRepresentationType

    # -------------------------------------------------------------------------
    #   imageRepresentation:
    #
    #   Give the path representation to the image browser.
    # -------------------------------------------------------------------------
    def imageRepresentation(self):
        return self._path

    # -------------------------------------------------------------------------
    #   imageUID:
    #
    #   Use the absolute file path as the identifier.
    # -------------------------------------------------------------------------
    def imageUID(self):
        return self._path


class ImageBrowserController(Cocoa.NSWindowController):
    imageBrowser = objc.IBOutlet()

    images = objc.ivar()
    importedImages = objc.ivar()

    # -------------------------------------------------------------------------
    #   awakeFromNib:
    # -------------------------------------------------------------------------
    def awakeFromNib(self):
        # Create two arrays : The first is for the data source representation.
        # The second one contains temporary imported images  for thread safeness.
        self.images = Cocoa.NSMutableArray.alloc().init()
        self.importedImages = Cocoa.NSMutableArray.alloc().init()

        # Allow reordering, animations and set the dragging destination
        # delegate.
        self.imageBrowser.setAllowsReordering_(True)
        self.imageBrowser.setAnimates_(True)
        self.imageBrowser.setDraggingDestinationDelegate_(self)

    # -------------------------------------------------------------------------
    #   updateDatasource:
    #
    #   This is the entry point for reloading image browser data and
    #   triggering setNeedsDisplay.
    # -------------------------------------------------------------------------
    def updateDatasource(self):
        # Update the datasource, add recently imported items.
        self.images.extend(self.importedImages)

        # Empty the temporary array.
        del self.importedImages[:]

        # Reload the image browser, which triggers setNeedsDisplay.
        self.imageBrowser.reloadData()

    # -------------------------------------------------------------------------
    #   isImageFile:filePath
    #
    #   This utility method indicates if the file located at 'filePath' is
    #   an image file based on the UTI. It relies on the ImageIO framework for
    #   the supported type identifiers.
    #
    # -------------------------------------------------------------------------
    def isImageFile_(self, filePath):
        isImageFile = False
        uti = None

        url = Cocoa.CFURLCreateWithFileSystemPath(
            None, filePath, Cocoa.kCFURLPOSIXPathStyle, False
        )

        res, info = LaunchServices.LSCopyItemInfoForURL(
            url,
            LaunchServices.kLSRequestExtension | LaunchServices.kLSRequestTypeCreator,
            None,
        )
        if res == 0:
            # Obtain the UTI using the file information.

            # If there is a file extension, get the UTI.
            if info[3] is not None:
                uti = LaunchServices.UTTypeCreatePreferredIdentifierForTag(
                    LaunchServices.kUTTagClassFilenameExtension,
                    info[3],
                    LaunchServices.kUTTypeData,
                )

            # No UTI yet
            if uti is None:
                # If there is an OSType, get the UTI.
                typeString = LaunchServices.UTCreateStringForOSType(info.filetype)
                if typeString is not None:
                    uti = LaunchServices.UTTypeCreatePreferredIdentifierForTag(
                        LaunchServices.kUTTagClassOSType,
                        typeString,
                        LaunchServices.kUTTypeData,
                    )

            # Verify that this is a file that the ImageIO framework supports.
            if uti is not None:
                supportedTypes = Quartz.CGImageSourceCopyTypeIdentifiers()

                for item in supportedTypes:
                    if LaunchServices.UTTypeConformsTo(uti, item):
                        isImageFile = True
                        break

        return isImageFile

    # -------------------------------------------------------------------------
    #   addAnImageWithPath:path
    # -------------------------------------------------------------------------
    def addAnImageWithPath_(self, path):
        addObject = False

        fileAttribs = Cocoa.NSFileManager.defaultManager().fileAttributesAtPath_traverseLink_(
            path, True
        )
        if fileAttribs is not None:
            # Check for packages.
            if Cocoa.NSFileTypeDirectory == fileAttribs[Cocoa.NSFileType]:
                if not Cocoa.NSWorkspace.sharedWorkspace().isFilePackageAtPath_(path):
                    addObject = True  # If it is a file, it's OK to add.

            else:
                addObject = True  # It is a file, so it's OK to add.

        if addObject and self.isImageFile_(path):
            # Add a path to the temporary images array.
            p = myImageObject.alloc().init()
            p.setPath_(path)
            self.importedImages.append(p)

    # -------------------------------------------------------------------------
    #   addImagesWithPath:path:recursive
    # -------------------------------------------------------------------------
    def addImagesWithPath_recursive_(self, path, recursive):
        if os.path.isdir(path):
            content = os.listdir(path)
            # Parse the directory content.
            for fn in content:
                if recursive:
                    self.addImagesWithPath_recursive_(os.path.join(path, fn), True)
                else:
                    self.addAnImageWithPath_(os.path.join(path, fn))

        else:
            self.addAnImageWithPath_(path)

    # -------------------------------------------------------------------------
    #   addImagesWithPaths:paths
    #
    #   Performed in an independent thread, parse all paths in "paths" and
    #   add these paths in the temporary images array.
    # -------------------------------------------------------------------------
    def addImagesWithPaths_(self, paths):
        pool = Cocoa.NSAutoreleasePool.alloc().init()

        for path in paths:
            isdir = os.path.isdir(path)
            self.addImagesWithPath_recursive_(path, isdir)

        # Update the data source in the main thread.
        self.performSelectorOnMainThread_withObject_waitUntilDone_(
            b"updateDatasource", None, True
        )

        del pool

    # pragma mark -
    # pragma mark actions

    # -------------------------------------------------------------------------
    #   addImageButtonClicked:sender
    #
    #   The user clicked the Add button.d
    # -------------------------------------------------------------------------
    @objc.IBAction
    def addImageButtonClicked_(self, sender):
        path = openFiles()
        if path:
            # launch import in an independent thread
            Cocoa.NSThread.detachNewThreadSelector_toTarget_withObject_(
                b"addImagesWithPaths:", self, path
            )

    # -------------------------------------------------------------------------
    #   addImageButtonClicked:sender
    #
    #   Action called when the zoom slider changes.
    # -------------------------------------------------------------------------
    @objc.IBAction
    def zoomSliderDidChange_(self, sender):
        # update the zoom value to scale images
        self.imageBrowser.setZoomValue_(sender.floatValue())

        # redisplay
        self.imageBrowser.setNeedsDisplay_(True)

    # Implement the image browser  data source protocol .
    # The data source representation is a simple mutable array.

    # -------------------------------------------------------------------------
    #   numberOfItemsInImageBrowser:view
    # -------------------------------------------------------------------------
    def numberOfItemsInImageBrowser_(self, view):
        # The item count to display is the datadsource item count.
        return len(self.images)

    # -------------------------------------------------------------------------
    #   imageBrowser:view:index:
    # -------------------------------------------------------------------------
    def imageBrowser_itemAtIndex_(self, view, index):
        return self.images[index]

    # Implement some optional methods of the image browser  datasource protocol
    # to allow for removing and reodering items.

    # -------------------------------------------------------------------------
    #   removeItemsAtIndexes:
    #
    #   The user wants to delete images, so remove these entries from the data source.
    # -------------------------------------------------------------------------
    def imageBrowser_removeItemsAtIndexes_(self, view, indexes):
        self.images.removeObjectsAtIndexes_(indexes)

    # -------------------------------------------------------------------------
    #   moveItemsAtIndexes:
    #
    #   The user wants to reorder images, update the datadsource and the browser
    #   will reflect our changes.
    # -------------------------------------------------------------------------
    def imageBrowser_moveItemsAtIndexes_toIndex_(
        self, browser, indexes, destinationIndex
    ):
        temporaryArray = []

        # First remove items from the data source and keep them in a
        # temporary array.
        for index in sorted(indexes, reverse=True):
            if index < destinationIndex:
                destinationIndex -= 1

            obj = self.images[index]
            temporaryArray.append(obj)
            del self.images[index]

        # Then insert the removed items at the appropriate location.
        for item in temporaryArray:
            self.images.insertObject_atIndex_(item, destinationIndex)

        return True

    # -------------------------------------------------------------------------
    #   draggingEntered:sender
    # -------------------------------------------------------------------------
    def draggingEntered_(self, sender):
        return Cocoa.NSDragOperationCopy

    # -------------------------------------------------------------------------
    #   draggingUpdated:sender
    # -------------------------------------------------------------------------
    def draggingUpdated_(self, sender):
        return Cocoa.NSDragOperationCopy

    # -------------------------------------------------------------------------
    #   performDragOperation:sender
    # -------------------------------------------------------------------------
    def performDragOperation_(self, sender):
        pasteboard = sender.draggingPasteboard()

        # Look for paths on the pasteboard.
        data = None
        if Cocoa.NSFilenamesPboardType in pasteboard.types():
            data = pasteboard.dataForType_(Cocoa.NSFilenamesPboardType)

        if data is not None:
            # Retrieve  paths.
            (
                filenames,
                plformat,
                errorDescription,
            ) = Cocoa.NSPropertyListSerialization.propertyListFromData_mutabilityOption_format_errorDescription_(  # noqa: B950
                data, Cocoa.kCFPropertyListImmutable, None, None
            )

            # Add paths to the data source.
            for fn in filenames:
                self.addAnImageWithPath_(fn)

            # Make the image browser reload the data source.
            self.updateDatasource()

        # Accept the drag operation.
        return True



import math

import Cocoa
import Quartz


class ExportInfo(object):
    command = None
    fileType = None
    useQTForExport = False
    dpi = 1


def DEGREES_TO_RADIANS(degrees):
    return degrees * math.pi / 180


gScalingFactor = 1.0


# These routines are used for getting the correct results when
# drawing shadows and patterns with Quartz and exporting the
# results as bits. This is a hack; in principle the scaling
# factor should be passed to the draw proc.
def setScalingFactor(scalingFactor):
    global gScalingFactor
    if scalingFactor > 0:
        gScalingFactor = scalingFactor


def getScalingFactor():
    return gScalingFactor


_appBundle = None


def getAppBundle():
    global _appBundle

    if _appBundle is None:
        _appBundle = Cocoa.CFBundleGetMainBundle()

    return _appBundle


#
#    This version of getTheRGBColorSpace returns
#    the DeviceRGB color space.
#
_deviceRGB = None


def getTheRGBColorSpace():
    global _deviceRGB

    # Set once, the first time this function is called.
    if _deviceRGB is None:
        _deviceRGB = Quartz.CGColorSpaceCreateDeviceRGB()

    return _deviceRGB


_genericRGBColorSpace = None


def getTheCalibratedRGBColorSpace():
    global _genericRGBColorSpace

    if _genericRGBColorSpace is None:
        _genericRGBColorSpace = Quartz.CGColorSpaceCreateWithName(
            Quartz.kCGColorSpaceGenericRGB
        )

    return _genericRGBColorSpace


_genericGrayColorSpace = None


def getTheCalibratedGrayColorSpace():
    global _genericGrayColorSpace

    if _genericGrayColorSpace is None:
        _genericGrayColorSpace = Quartz.CGColorSpaceCreateWithName(
            Quartz.kCGColorSpaceGenericGray
        )
    return _genericGrayColorSpace


_genericSRGBColorSpace = None


def getTheSRGBColorSpace():
    # This only works on 10.5 or later
    global _genericSRGBColorSpace

    if _genericSRGBColorSpace is None:
        _genericSRGBColorSpace = Quartz.CGColorSpaceCreateWithName(
            Quartz.kCGColorSpaceGenericRGB
        )  # XXX: should be GenericSRGB
    return _genericSRGBColorSpace


def getTheDisplayColorSpace():
    # This is a hack, basicly here because the C implementation uses APIs that
    # aren't wrapped yet.
    return getTheRGBColorSpace()


_rgbWhite = None


def getRGBOpaqueWhiteColor():
    global _rgbWhite

    if _rgbWhite is None:
        opaqueWhite = (1.0, 1.0, 1.0, 1.0)
        _rgbWhite = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueWhite)
    return _rgbWhite


_rgbBlack = None


def getRGBOpaqueBlackColor():
    global _rgbBlack
    if _rgbBlack is None:
        opaqueBlack = (0, 0, 0, 1)
        _rgbBlack = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueBlack)
    return _rgbBlack


_rgbGray = None


def getRGBOpaqueGrayColor():
    global _rgbGray
    if _rgbGray is None:
        opaqueGray = (0.9, 0.9, 0.9, 1)
        _rgbGray = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueGray)
    return _rgbGray


_rgbRed = None


def getRGBOpaqueRedColor():
    global _rgbRed
    if _rgbRed is None:
        opaqueRed = (0.663, 0, 0.031, 1)
        _rgbRed = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueRed)
    return _rgbRed


_rgbBlue = None


def getRGBOpaqueBlueColor():
    global _rgbBlue
    if _rgbBlue is None:
        opaqueBlue = (0.482, 0.62, 0.871, 1)
        _rgbBlue = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueBlue)
    return _rgbBlue


_rgbPurple = None


def getRGBOpaquePurpleColor():
    global _rgbPurple
    if _rgbPurple is None:
        opaquePurple = (0.69, 0.486, 0.722, 1)
        _rgbPurple = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaquePurple)
    return _rgbPurple


_rgbDarkBlue = None


def getRGBOpaqueDarkBlueColor():
    global _rgbDarkBlue
    if _rgbDarkBlue is None:
        opaqueDarkBlue = (0.11, 0.208, 0.451, 1)
        _rgbDarkBlue = Quartz.CGColorCreate(
            getTheCalibratedRGBColorSpace(), opaqueDarkBlue
        )
    return _rgbDarkBlue


_rgbBrown = None


def getRGBOpaqueBrownColor():
    global _rgbBrown
    if _rgbBrown is None:
        opaqueBrown = (0.325, 0.208, 0.157, 1)
        _rgbBrown = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueBrown)
    return _rgbBrown


_rgbOrange = None


def getRGBOpaqueOrangeColor():
    global _rgbOrange
    if _rgbOrange is None:
        opaqueOrange = (0.965, 0.584, 0.059, 1)
        _rgbOrange = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueOrange)
    return _rgbOrange


_rgbYellow = None


def getRGBOpaqueYellowColor():
    global _rgbYellow
    if _rgbYellow is None:
        opaqueYellow = (1, 0.816, 0, 1)
        _rgbYellow = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueYellow)
    return _rgbYellow


_rgbGreen = None


def getRGBOpaqueGreenColor():
    global _rgbGreen
    if _rgbGreen is None:
        opaqueGreen = (0.584, 0.871, 0.318, 1)
        _rgbGreen = Quartz.CGColorCreate(getTheCalibratedRGBColorSpace(), opaqueGreen)
    return _rgbGreen


_rgbDarkGreen = None


def getRGBOpaqueDarkGreenColor():
    global _rgbDarkGreen
    if _rgbDarkGreen is None:
        opaqueDarkGreen = (0.404, 0.808, 0.239, 1)
        _rgbDarkGreen = Quartz.CGColorCreate(
            getTheCalibratedRGBColorSpace(), opaqueDarkGreen
        )
    return _rgbDarkGreen


def myCGContextAddEllipseInRect(context, r):
    if hasattr(Quartz, "CGContextAddEllipseInRect"):
        Quartz.CGContextAddEllipseInRect(context, r)

    else:
        # This is not a perfect emulation but is correct as long as there is
        # not an open subpath already in the current path. In that case the
        # CGContextClosePath here would not necessarily produce the desired
        # result.
        Quartz.CGContextSaveGState(context)
        if 1:
            # Translate to the center of the ellipse.
            Quartz.CGContextTranslateCTM(
                context, Quartz.CGRectGetMidX(r), Quartz.CGRectGetMidY(r)
            )
            # Scale by half the width and height of the rectangle
            # bounding the ellipse.
            Quartz.CGContextScaleCTM(context, r.size.width / 2, r.size.height / 2)
            # Establish a current point at the first point
            # on the ellipse. This ensures that there
            # is no line segment connecting the previous
            # current point to the first point on the subpath.
            Quartz.CGContextMoveToPoint(context, 1, 0)
            # Circular arc around the ellipse center with
            # a radius that, when scaled by the CTM, produces
            # the major and minor axes of the ellipse. Since
            # CGContextAddEllipseInRect defines the direction
            # of the path as clockwise, this routine will
            # draw the arc clockwise also.
            Quartz.CGContextAddArc(context, 0, 0, 1, 0, 2 * math.pi, 1)
            Quartz.CGContextClosePath(context)
        Quartz.CGContextRestoreGState(context)


# Routines that are useful for debugging.


def drawPoint(context, p):
    Quartz.CGContextSaveGState(context)
    if 1:
        # Opaque black.
        Quartz.CGContextSetRGBStrokeColor(context, 0, 0, 0, 1)
        Quartz.CGContextSetLineWidth(context, 5)
        Quartz.CGContextSetLineCap(context, Quartz.kCGLineCapRound)
        Quartz.CGContextMoveToPoint(context, p.x, p.y)
        Quartz.CGContextAddLineToPoint(context, p.x, p.y)
        Quartz.CGContextStrokePath(context)
    Quartz.CGContextRestoreGState(context)


def printCTM(context):
    t = Quartz.CGContextGetCTM(context)
    print("CurrentCTM is %r" % (t,))


kTickLength = 5
kTickDistance = 72
kAxesLength = 20 * kTickDistance


def drawCoordinateAxes(context):
    tickLength = kTickLength

    Quartz.CGContextSaveGState(context)
    if 1:
        Quartz.CGContextBeginPath(context)
        # Paint the x-axis in red.
        Quartz.CGContextSetRGBStrokeColor(context, 1, 0, 0, 1)
        Quartz.CGContextMoveToPoint(context, -kTickLength, 0.0)
        Quartz.CGContextAddLineToPoint(context, kAxesLength, 0.0)
        Quartz.CGContextDrawPath(context, Quartz.kCGPathStroke)

        # Paint the y-axis in blue.
        Quartz.CGContextSetRGBStrokeColor(context, 0, 0, 1, 1)
        Quartz.CGContextMoveToPoint(context, 0, -kTickLength)
        Quartz.CGContextAddLineToPoint(context, 0, kAxesLength)
        Quartz.CGContextDrawPath(context, Quartz.kCGPathStroke)

        # Paint the x-axis tick marks in red.
        Quartz.CGContextSetRGBStrokeColor(context, 1, 0, 0, 1)
        for _ in range(2):
            for t in range(0, kAxesLength, kTickDistance):
                Quartz.CGContextMoveToPoint(context, t, -tickLength)
                Quartz.CGContextAddLineToPoint(context, t, tickLength)

            Quartz.CGContextDrawPath(context, Quartz.kCGPathStroke)
            Quartz.CGContextRotateCTM(context, math.pi / 2.0)
            # Paint the y-axis tick marks in blue.
            Quartz.CGContextSetRGBStrokeColor(context, 0, 0, 1, 1)

        drawPoint(context, Quartz.CGPointZero)
    Quartz.CGContextRestoreGState(context)


def drawDebuggingRect(context, rect):
    Quartz.CGContextSaveGState(context)
    if 1:
        Quartz.CGContextSetLineWidth(context, 4.0)
        # Draw opaque red from top-left to bottom-right.
        Quartz.CGContextSetRGBStrokeColor(context, 1, 0, 0, 1.0)
        Quartz.CGContextMoveToPoint(
            context, rect.origin.x, rect.origin.y + rect.size.height
        )
        Quartz.CGContextAddLineToPoint(
            context, rect.origin.x + rect.size.width, rect.origin.y
        )
        Quartz.CGContextStrokePath(context)
        # Draw opaque blue from top-right to bottom-left.
        Quartz.CGContextSetRGBStrokeColor(context, 0, 0, 1, 1.0)
        Quartz.CGContextMoveToPoint(
            context, rect.origin.x + rect.size.width, rect.origin.y + rect.size.height
        )
        Quartz.CGContextAddLineToPoint(context, rect.origin.x, rect.origin.y)
        Quartz.CGContextStrokePath(context)
        # Opaque black.
        Quartz.CGContextSetRGBStrokeColor(context, 0, 0, 0, 1.0)
        Quartz.CGContextStrokeRect(context, rect)
    Quartz.CGContextRestoreGState(context)



import AppDrawing
import Cocoa
import objc
import UIHandling

_drawingCommand = UIHandling.kCommandStrokedAndFilledRects
_pdfDocument = None


class MyView(Cocoa.NSView):
    currentMenuItem = objc.IBOutlet()

    def drawRect_(self, rect):
        context = Cocoa.NSGraphicsContext.currentContext().graphicsPort()
        AppDrawing.myDispatchDrawing(context, _drawingCommand)

    @objc.IBAction
    def setDrawCommand_(self, sender):
        global _drawingCommand

        newCommand = sender.tag()

        if newCommand != _drawingCommand:
            _drawingCommand = newCommand
            self.setNeedsDisplay_(True)

            self.currentMenuItem.setState_(Cocoa.NSOffState)
            self.currentMenuItem = sender
            self.currentMenuItem.setState_(Cocoa.NSOnState)

    def currentPrintableCommand(self):
        # The best representation for printing or exporting
        # when the current command caches using a bitmap context
        # or a layer is to not do any caching.
        if _drawingCommand == UIHandling.kCommandDoCGLayer:
            return UIHandling.kCommandDoUncachedDrawing

        return _drawingCommand

    def print_(self, sender):
        global _drawingCommand

        savedDrawingCommand = _drawingCommand
        _drawingCommand = self.currentPrintableCommand()
        Cocoa.NSPrintOperation.printOperationWithView_(self).runOperation()
        _drawingCommand = savedDrawingCommand

    def knowsPageRange_(self, page_range):
        return True, Cocoa.NSRange(1, 1)

    def rectForPage_(self, page):
        pi = Cocoa.NSPrintOperation.currentOperation().printInfo()
        paperSize = pi.paperSize()
        return Cocoa.NSMakeRect(0, 0, paperSize.width, paperSize.height)

    def validateMenuItem_(self, menuItem):
        if menuItem.tag() == _drawingCommand:
            self.currentMenuItem = menuItem
            menuItem.setState_(True)
        else:
            menuItem.setState_(False)

        return True



# This file is generated by objective.metadata
#
# Last update: Fri Aug  9 18:36:47 2019
#
# flake8: noqa

import sys

import objc

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
misc.update(
    {
        "CVTimeStamp": objc.createStructType(
            "CVTimeStamp",
            sel32or64(
                b"{_CVTimeStamp=IiqQdq{CVSMPTETime=ssLLLssss}QQ}",
                b"{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
            ),
            [
                "version",
                "videoTimeScale",
                "videoTime",
                "hostTime",
                "rateScalar",
                "videoRefreshPeriod",
                "smpteTime",
                "flags",
                "reserved",
            ],
        ),
        "CVPlanarPixelBufferInfo_YCbCrBiPlanar": objc.createStructType(
            "CVPlanarPixelBufferInfo_YCbCrBiPlanar",
            b"{CVPlanarPixelBufferInfo_YCbCrBiPlanar={CVPlanarComponentInfo=iI}{CVPlanarComponentInfo=iI}}",
            ["componentInfoY", "componentInfoCbCr"],
        ),
        "CVPlanarPixelBufferInfo_YCbCrPlanar": objc.createStructType(
            "CVPlanarPixelBufferInfo_YCbCrPlanar",
            b"{CVPlanarPixelBufferInfo_YCbCrPlanar={CVPlanarComponentInfo=iI}{CVPlanarComponentInfo=iI}{CVPlanarComponentInfo=iI}}",
            ["componentInfoY", "componentInfoCb", "componentInfoCr"],
        ),
        "CVPlanarComponentInfo": objc.createStructType(
            "CVPlanarComponentInfo",
            b"{CVPlanarComponentInfo=iI}",
            ["offset", "rowBytes"],
        ),
        "CVTime": objc.createStructType(
            "CVTime", b"{_CVTime=qii}", ["timeValue", "timeScale", "flags"]
        ),
        "CVSMPTETime": objc.createStructType(
            "CVSMPTETime",
            sel32or64(b"{CVSMPTETime=ssLLLssss}", b"{CVSMPTETime=ssIIIssss}"),
            [
                "subframes",
                "subframeDivisor",
                "counter",
                "type",
                "flags",
                "hours",
                "minutes",
                "seconds",
                "frames",
            ],
        ),
        "CVPlanarPixelBufferInfo": objc.createStructType(
            "CVPlanarPixelBufferInfo",
            b"{CVPlanarPixelBufferInfo=[1{CVPlanarComponentInfo=iI}]}",
            ["componentInfo"],
        ),
    }
)
constants = """$kCVBufferMovieTimeKey@^{__CFString=}$kCVBufferNonPropagatedAttachmentsKey@^{__CFString=}$kCVBufferPropagatedAttachmentsKey@^{__CFString=}$kCVBufferTimeScaleKey@^{__CFString=}$kCVBufferTimeValueKey@^{__CFString=}$kCVImageBufferAlphaChannelIsOpaque@^{__CFString=}$kCVImageBufferAlphaChannelModeKey@^{__CFString=}$kCVImageBufferAlphaChannelMode_PremultipliedAlpha@^{__CFString=}$kCVImageBufferAlphaChannelMode_StraightAlpha@^{__CFString=}$kCVImageBufferCGColorSpaceKey@^{__CFString=}$kCVImageBufferChromaLocationBottomFieldKey@^{__CFString=}$kCVImageBufferChromaLocationTopFieldKey@^{__CFString=}$kCVImageBufferChromaLocation_Bottom@^{__CFString=}$kCVImageBufferChromaLocation_BottomLeft@^{__CFString=}$kCVImageBufferChromaLocation_Center@^{__CFString=}$kCVImageBufferChromaLocation_DV420@^{__CFString=}$kCVImageBufferChromaLocation_Left@^{__CFString=}$kCVImageBufferChromaLocation_Top@^{__CFString=}$kCVImageBufferChromaLocation_TopLeft@^{__CFString=}$kCVImageBufferChromaSubsamplingKey@^{__CFString=}$kCVImageBufferChromaSubsampling_411@^{__CFString=}$kCVImageBufferChromaSubsampling_420@^{__CFString=}$kCVImageBufferChromaSubsampling_422@^{__CFString=}$kCVImageBufferCleanApertureHeightKey@^{__CFString=}$kCVImageBufferCleanApertureHorizontalOffsetKey@^{__CFString=}$kCVImageBufferCleanApertureKey@^{__CFString=}$kCVImageBufferCleanApertureVerticalOffsetKey@^{__CFString=}$kCVImageBufferCleanApertureWidthKey@^{__CFString=}$kCVImageBufferColorPrimariesKey@^{__CFString=}$kCVImageBufferColorPrimaries_DCI_P3@^{__CFString=}$kCVImageBufferColorPrimaries_EBU_3213@^{__CFString=}$kCVImageBufferColorPrimaries_ITU_R_2020@^{__CFString=}$kCVImageBufferColorPrimaries_ITU_R_709_2@^{__CFString=}$kCVImageBufferColorPrimaries_P22@^{__CFString=}$kCVImageBufferColorPrimaries_P3_D65@^{__CFString=}$kCVImageBufferColorPrimaries_SMPTE_C@^{__CFString=}$kCVImageBufferContentLightLevelInfoKey@^{__CFString=}$kCVImageBufferDisplayDimensionsKey@^{__CFString=}$kCVImageBufferDisplayHeightKey@^{__CFString=}$kCVImageBufferDisplayWidthKey@^{__CFString=}$kCVImageBufferFieldCountKey@^{__CFString=}$kCVImageBufferFieldDetailKey@^{__CFString=}$kCVImageBufferFieldDetailSpatialFirstLineEarly@^{__CFString=}$kCVImageBufferFieldDetailSpatialFirstLineLate@^{__CFString=}$kCVImageBufferFieldDetailTemporalBottomFirst@^{__CFString=}$kCVImageBufferFieldDetailTemporalTopFirst@^{__CFString=}$kCVImageBufferGammaLevelKey@^{__CFString=}$kCVImageBufferICCProfileKey@^{__CFString=}$kCVImageBufferMasteringDisplayColorVolumeKey@^{__CFString=}$kCVImageBufferPixelAspectRatioHorizontalSpacingKey@^{__CFString=}$kCVImageBufferPixelAspectRatioKey@^{__CFString=}$kCVImageBufferPixelAspectRatioVerticalSpacingKey@^{__CFString=}$kCVImageBufferPreferredCleanApertureKey@^{__CFString=}$kCVImageBufferTransferFunctionKey@^{__CFString=}$kCVImageBufferTransferFunction_EBU_3213@^{__CFString=}$kCVImageBufferTransferFunction_ITU_R_2020@^{__CFString=}$kCVImageBufferTransferFunction_ITU_R_2100_HLG@^{__CFString=}$kCVImageBufferTransferFunction_ITU_R_709_2@^{__CFString=}$kCVImageBufferTransferFunction_Linear@^{__CFString=}$kCVImageBufferTransferFunction_SMPTE_240M_1995@^{__CFString=}$kCVImageBufferTransferFunction_SMPTE_C@^{__CFString=}$kCVImageBufferTransferFunction_SMPTE_ST_2084_PQ@^{__CFString=}$kCVImageBufferTransferFunction_SMPTE_ST_428_1@^{__CFString=}$kCVImageBufferTransferFunction_UseGamma@^{__CFString=}$kCVImageBufferTransferFunction_sRGB@^{__CFString=}$kCVImageBufferYCbCrMatrixKey@^{__CFString=}$kCVImageBufferYCbCrMatrix_DCI_P3@^{__CFString=}$kCVImageBufferYCbCrMatrix_ITU_R_2020@^{__CFString=}$kCVImageBufferYCbCrMatrix_ITU_R_601_4@^{__CFString=}$kCVImageBufferYCbCrMatrix_ITU_R_709_2@^{__CFString=}$kCVImageBufferYCbCrMatrix_P3_D65@^{__CFString=}$kCVImageBufferYCbCrMatrix_SMPTE_240M_1995@^{__CFString=}$kCVIndefiniteTime@{_CVTime=qii}$kCVMetalTextureCacheMaximumTextureAgeKey@^{__CFString=}$kCVMetalTextureStorageMode@^{__CFString=}$kCVMetalTextureUsage@^{__CFString=}$kCVOpenGLBufferHeight@^{__CFString=}$kCVOpenGLBufferInternalFormat@^{__CFString=}$kCVOpenGLBufferMaximumMipmapLevel@^{__CFString=}$kCVOpenGLBufferPoolMaximumBufferAgeKey@^{__CFString=}$kCVOpenGLBufferPoolMinimumBufferCountKey@^{__CFString=}$kCVOpenGLBufferTarget@^{__CFString=}$kCVOpenGLBufferWidth@^{__CFString=}$kCVOpenGLTextureCacheChromaSamplingModeAutomatic@^{__CFString=}$kCVOpenGLTextureCacheChromaSamplingModeBestPerformance@^{__CFString=}$kCVOpenGLTextureCacheChromaSamplingModeHighestQuality@^{__CFString=}$kCVOpenGLTextureCacheChromaSamplingModeKey@^{__CFString=}$kCVPixelBufferBytesPerRowAlignmentKey@^{__CFString=}$kCVPixelBufferCGBitmapContextCompatibilityKey@^{__CFString=}$kCVPixelBufferCGImageCompatibilityKey@^{__CFString=}$kCVPixelBufferExtendedPixelsBottomKey@^{__CFString=}$kCVPixelBufferExtendedPixelsLeftKey@^{__CFString=}$kCVPixelBufferExtendedPixelsRightKey@^{__CFString=}$kCVPixelBufferExtendedPixelsTopKey@^{__CFString=}$kCVPixelBufferHeightKey@^{__CFString=}$kCVPixelBufferIOSurfaceCoreAnimationCompatibilityKey@^{__CFString=}$kCVPixelBufferIOSurfaceOpenGLESFBOCompatibilityKey@^{__CFString=}$kCVPixelBufferIOSurfaceOpenGLESTextureCompatibilityKey@^{__CFString=}$kCVPixelBufferIOSurfaceOpenGLFBOCompatibilityKey@^{__CFString=}$kCVPixelBufferIOSurfaceOpenGLTextureCompatibilityKey@^{__CFString=}$kCVPixelBufferIOSurfacePropertiesKey@^{__CFString=}$kCVPixelBufferMemoryAllocatorKey@^{__CFString=}$kCVPixelBufferMetalCompatibilityKey@^{__CFString=}$kCVPixelBufferOpenGLCompatibilityKey@^{__CFString=}$kCVPixelBufferOpenGLESCompatibilityKey@^{__CFString=}$kCVPixelBufferOpenGLESTextureCacheCompatibilityKey@^{__CFString=}$kCVPixelBufferOpenGLTextureCacheCompatibilityKey@^{__CFString=}$kCVPixelBufferPixelFormatTypeKey@^{__CFString=}$kCVPixelBufferPlaneAlignmentKey@^{__CFString=}$kCVPixelBufferPoolAllocationThresholdKey@^{__CFString=}$kCVPixelBufferPoolFlushExcessBuffers$kCVPixelBufferPoolFreeBufferNotification@^{__CFString=}$kCVPixelBufferPoolMaximumBufferAgeKey@^{__CFString=}$kCVPixelBufferPoolMinimumBufferCountKey@^{__CFString=}$kCVPixelBufferWidthKey@^{__CFString=}$kCVPixelFormatBitsPerBlock@^{__CFString=}$kCVPixelFormatBlackBlock@^{__CFString=}$kCVPixelFormatBlockHeight@^{__CFString=}$kCVPixelFormatBlockHorizontalAlignment@^{__CFString=}$kCVPixelFormatBlockVerticalAlignment@^{__CFString=}$kCVPixelFormatBlockWidth@^{__CFString=}$kCVPixelFormatCGBitmapContextCompatibility@^{__CFString=}$kCVPixelFormatCGBitmapInfo@^{__CFString=}$kCVPixelFormatCGImageCompatibility@^{__CFString=}$kCVPixelFormatCodecType@^{__CFString=}$kCVPixelFormatComponentRange@^{__CFString=}$kCVPixelFormatComponentRange_FullRange@^{__CFString=}$kCVPixelFormatComponentRange_VideoRange@^{__CFString=}$kCVPixelFormatComponentRange_WideRange@^{__CFString=}$kCVPixelFormatConstant@^{__CFString=}$kCVPixelFormatContainsAlpha@^{__CFString=}$kCVPixelFormatContainsGrayscale@^{__CFString=}$kCVPixelFormatContainsRGB@^{__CFString=}$kCVPixelFormatContainsYCbCr@^{__CFString=}$kCVPixelFormatFillExtendedPixelsCallback@^{__CFString=}$kCVPixelFormatFourCC@^{__CFString=}$kCVPixelFormatHorizontalSubsampling@^{__CFString=}$kCVPixelFormatName@^{__CFString=}$kCVPixelFormatOpenGLCompatibility@^{__CFString=}$kCVPixelFormatOpenGLESCompatibility@^{__CFString=}$kCVPixelFormatOpenGLFormat@^{__CFString=}$kCVPixelFormatOpenGLInternalFormat@^{__CFString=}$kCVPixelFormatOpenGLType@^{__CFString=}$kCVPixelFormatPlanes@^{__CFString=}$kCVPixelFormatQDCompatibility@^{__CFString=}$kCVPixelFormatVerticalSubsampling@^{__CFString=}$kCVZeroTime@{_CVTime=qii}$"""
enums = """$COREVIDEO_INCLUDED_IOSURFACE_HEADER_FILE@1$kCVAttachmentMode_ShouldNotPropagate@0$kCVAttachmentMode_ShouldPropagate@1$kCVPixelBufferLock_ReadOnly@1$kCVPixelBufferPoolFlushExcessBuffers@1$kCVPixelFormatType_128RGBAFloat@1380410945$kCVPixelFormatType_14Bayer_BGGR@1650943796$kCVPixelFormatType_14Bayer_GBRG@1734505012$kCVPixelFormatType_14Bayer_GRBG@1735549492$kCVPixelFormatType_14Bayer_RGGB@1919379252$kCVPixelFormatType_16BE555@16$kCVPixelFormatType_16BE565@1110783541$kCVPixelFormatType_16Gray@1647392359$kCVPixelFormatType_16LE555@1278555445$kCVPixelFormatType_16LE5551@892679473$kCVPixelFormatType_16LE565@1278555701$kCVPixelFormatType_1IndexedGray_WhiteIsZero@33$kCVPixelFormatType_1Monochrome@1$kCVPixelFormatType_24BGR@842285639$kCVPixelFormatType_24RGB@24$kCVPixelFormatType_2Indexed@2$kCVPixelFormatType_2IndexedGray_WhiteIsZero@34$kCVPixelFormatType_30RGB@1378955371$kCVPixelFormatType_30RGBLEPackedWideGamut@1999843442$kCVPixelFormatType_32ABGR@1094862674$kCVPixelFormatType_32ARGB@32$kCVPixelFormatType_32AlphaGray@1647522401$kCVPixelFormatType_32BGRA@1111970369$kCVPixelFormatType_32RGBA@1380401729$kCVPixelFormatType_420YpCbCr10BiPlanarFullRange@2019963440$kCVPixelFormatType_420YpCbCr10BiPlanarVideoRange@2016686640$kCVPixelFormatType_420YpCbCr8BiPlanarFullRange@875704422$kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange@875704438$kCVPixelFormatType_420YpCbCr8Planar@2033463856$kCVPixelFormatType_420YpCbCr8PlanarFullRange@1714696752$kCVPixelFormatType_420YpCbCr8VideoRange_8A_TriPlanar@1982882104$kCVPixelFormatType_422YpCbCr10@1983000880$kCVPixelFormatType_422YpCbCr10BiPlanarFullRange@2019963442$kCVPixelFormatType_422YpCbCr10BiPlanarVideoRange@2016686642$kCVPixelFormatType_422YpCbCr16@1983000886$kCVPixelFormatType_422YpCbCr8@846624121$kCVPixelFormatType_422YpCbCr8FullRange@2037741158$kCVPixelFormatType_422YpCbCr8_yuvs@2037741171$kCVPixelFormatType_422YpCbCr_4A_8BiPlanar@1630697081$kCVPixelFormatType_4444AYpCbCr16@2033463606$kCVPixelFormatType_4444AYpCbCr8@2033463352$kCVPixelFormatType_4444YpCbCrA8@1983131704$kCVPixelFormatType_4444YpCbCrA8R@1916022840$kCVPixelFormatType_444YpCbCr10@1983131952$kCVPixelFormatType_444YpCbCr10BiPlanarFullRange@2019963956$kCVPixelFormatType_444YpCbCr10BiPlanarVideoRange@2016687156$kCVPixelFormatType_444YpCbCr8@1983066168$kCVPixelFormatType_48RGB@1647589490$kCVPixelFormatType_4Indexed@4$kCVPixelFormatType_4IndexedGray_WhiteIsZero@36$kCVPixelFormatType_64ARGB@1647719521$kCVPixelFormatType_64RGBAHalf@1380411457$kCVPixelFormatType_8Indexed@8$kCVPixelFormatType_8IndexedGray_WhiteIsZero@40$kCVPixelFormatType_ARGB2101010LEPacked@1815162994$kCVPixelFormatType_DepthFloat16@1751410032$kCVPixelFormatType_DepthFloat32@1717855600$kCVPixelFormatType_DisparityFloat16@1751411059$kCVPixelFormatType_DisparityFloat32@1717856627$kCVPixelFormatType_OneComponent16Half@1278226536$kCVPixelFormatType_OneComponent32Float@1278226534$kCVPixelFormatType_OneComponent8@1278226488$kCVPixelFormatType_TwoComponent16Half@843264104$kCVPixelFormatType_TwoComponent32Float@843264102$kCVPixelFormatType_TwoComponent8@843264056$kCVReturnAllocationFailed@-6662$kCVReturnDisplayLinkAlreadyRunning@-6671$kCVReturnDisplayLinkCallbacksNotSet@-6673$kCVReturnDisplayLinkNotRunning@-6672$kCVReturnError@-6660$kCVReturnFirst@-6660$kCVReturnInvalidArgument@-6661$kCVReturnInvalidDisplay@-6670$kCVReturnInvalidPixelBufferAttributes@-6682$kCVReturnInvalidPixelFormat@-6680$kCVReturnInvalidPoolAttributes@-6691$kCVReturnInvalidSize@-6681$kCVReturnLast@-6699$kCVReturnPixelBufferNotMetalCompatible@-6684$kCVReturnPixelBufferNotOpenGLCompatible@-6683$kCVReturnPoolAllocationFailed@-6690$kCVReturnRetry@-6692$kCVReturnSuccess@0$kCVReturnUnsupported@-6663$kCVReturnWouldExceedAllocationThreshold@-6689$kCVSMPTETimeRunning@2$kCVSMPTETimeType24@0$kCVSMPTETimeType25@1$kCVSMPTETimeType2997@4$kCVSMPTETimeType2997Drop@5$kCVSMPTETimeType30@3$kCVSMPTETimeType30Drop@2$kCVSMPTETimeType5994@7$kCVSMPTETimeType60@6$kCVSMPTETimeValid@1$kCVTimeIsIndefinite@1$kCVTimeStampBottomField@131072$kCVTimeStampHostTimeValid@2$kCVTimeStampIsInterlaced@196608$kCVTimeStampRateScalarValid@16$kCVTimeStampSMPTETimeValid@4$kCVTimeStampTopField@65536$kCVTimeStampVideoHostTimeValid@3$kCVTimeStampVideoRefreshPeriodValid@8$kCVTimeStampVideoTimeValid@1$kReturnRetry@-6692$"""
misc.update({})
functions = {
    "CVImageBufferGetEncodedSize": (
        sel32or64(b"{CGSize=ff}^{__CVBuffer=}", b"{CGSize=dd}^{__CVBuffer=}"),
    ),
    "CVOpenGLTextureRelease": (b"v^{__CVBuffer=}",),
    "CVPixelBufferPoolRelease": (b"v^{__CVPixelBufferPool=}",),
    "CVPixelBufferPoolGetTypeID": (sel32or64(b"L", b"Q"),),
    "CVYCbCrMatrixGetIntegerCodePointForString": (b"i^{__CFString=}",),
    "CVPixelBufferCreate": (
        sel32or64(
            b"i^{__CFAllocator=}LLL^{__CFDictionary=}^^{__CVBuffer=}",
            b"i^{__CFAllocator=}QQI^{__CFDictionary=}^^{__CVBuffer=}",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CVOpenGLBufferPoolGetTypeID": (sel32or64(b"L", b"Q"),),
    "CVPixelBufferFillExtendedPixels": (b"i^{__CVBuffer=}",),
    "CVOpenGLTextureCacheRetain": (
        b"^{__CVOpenGLTextureCache=}^{__CVOpenGLTextureCache=}",
    ),
    "CVOpenGLBufferPoolCreateOpenGLBuffer": (
        b"i^{__CFAllocator=}^{__CVOpenGLBufferPool=}^^{__CVBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {2: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVDisplayLinkSetCurrentCGDisplay": (b"i^{__CVDisplayLink=}I",),
    "CVBufferSetAttachment": (b"v^{__CVBuffer=}^{__CFString=}@I",),
    "CVGetCurrentHostTime": (b"Q", "", {"variadic": False}),
    "CVPixelBufferPoolCreate": (
        b"i^{__CFAllocator=}^{__CFDictionary=}^{__CFDictionary=}^^{__CVPixelBufferPool=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {3: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVPixelBufferGetHeightOfPlane": (
        sel32or64(b"L^{__CVBuffer=}L", b"Q^{__CVBuffer=}Q"),
    ),
    "CVBufferRetain": (b"^{__CVBuffer=}^{__CVBuffer=}",),
    "CVDisplayLinkTranslateTime": (
        sel32or64(
            b"i^{__CVDisplayLink=}^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssLLLssss}QQ}^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssLLLssss}QQ}",
            b"i^{__CVDisplayLink=}^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
        ),
        "",
        {"arguments": {1: {"type_modifier": "n"}, 2: {"type_modifier": "o"}}},
    ),
    "CVPixelBufferRetain": (b"^{__CVBuffer=}^{__CVBuffer=}",),
    "CVPixelBufferGetPlaneCount": (sel32or64(b"L^{__CVBuffer=}", b"Q^{__CVBuffer=}"),),
    "CVOpenGLTextureCacheRelease": (b"v^{__CVOpenGLTextureCache=}",),
    "CVPixelBufferGetBaseAddress": (
        b"^v^{__CVBuffer=}",
        "",
        {"retval": {"c_array_of_variable_length": True}},
    ),
    "CVOpenGLBufferPoolRelease": (b"v^{__CVOpenGLBufferPool=}",),
    "CVPixelBufferLockBaseAddress": (b"i^{__CVBuffer=}Q",),
    "CVOpenGLTextureCacheGetTypeID": (sel32or64(b"L", b"Q"),),
    "CVPixelBufferUnlockBaseAddress": (b"i^{__CVBuffer=}Q",),
    "CVMetalTextureCacheCreateTextureFromImage": (
        b"i^{__CFAllocator=}^{__CVMetalTextureCache=}^{__CVBuffer=}^{__CFDictionary=}QQQQ^^{__CVBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {8: {"type_modifier": "o"}},
        },
    ),
    "CVOpenGLTextureIsFlipped": (b"Z^{__CVBuffer=}",),
    "CVMetalTextureCacheFlush": (b"v^{__CVMetalTextureCache=}Q",),
    "CVPixelBufferPoolCreatePixelBuffer": (
        b"i^{__CFAllocator=}^{__CVPixelBufferPool=}^^{__CVBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {2: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVPixelBufferGetTypeID": (sel32or64(b"L", b"Q"),),
    "CVDisplayLinkGetActualOutputVideoRefreshPeriod": (b"d^{__CVDisplayLink=}",),
    "CVPixelBufferGetWidth": (sel32or64(b"L^{__CVBuffer=}", b"Q^{__CVBuffer=}"),),
    "CVMetalTextureCacheGetTypeID": (b"Q",),
    "CVDisplayLinkCreateWithCGDisplay": (
        b"iI^^{__CVDisplayLink=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {1: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVBufferRelease": (b"v^{__CVBuffer=}",),
    "CVDisplayLinkStart": (b"i^{__CVDisplayLink=}",),
    "CVDisplayLinkGetCurrentTime": (
        sel32or64(
            b"i^{__CVDisplayLink=}^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssLLLssss}QQ}",
            b"i^{__CVDisplayLink=}^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
        ),
        "",
        {"arguments": {1: {"type_modifier": "o"}}},
    ),
    "CVPixelFormatDescriptionArrayCreateWithAllPixelFormatTypes": (
        b"^{__CFArray=}^{__CFAllocator=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CVPixelBufferPoolGetAttributes": (b"^{__CFDictionary=}^{__CVPixelBufferPool=}",),
    "CVBufferGetAttachments": (b"^{__CFDictionary=}^{__CVBuffer=}I",),
    "CVPixelFormatDescriptionRegisterDescriptionWithPixelFormatType": (
        sel32or64(b"v^{__CFDictionary=}L", b"v^{__CFDictionary=}I"),
    ),
    "CVOpenGLBufferPoolCreate": (
        b"i^{__CFAllocator=}^{__CFDictionary=}^{__CFDictionary=}^^{__CVOpenGLBufferPool=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {3: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVDisplayLinkRetain": (b"^{__CVDisplayLink=}^{__CVDisplayLink=}",),
    "CVPixelBufferCreateWithIOSurface": (
        b"i^{__CFAllocator=}^{__IOSurface=}^{__CFDictionary=}^^{__CVBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {3: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVDisplayLinkCreateWithOpenGLDisplayMask": (
        b"iI^^{__CVDisplayLink=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {1: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVDisplayLinkSetOutputHandler": (
        b"i^{__CVDisplayLink=}@?",
        "",
        {
            "arguments": {
                1: {
                    "callable": {
                        "retval": {"type": b"i"},
                        "arguments": {
                            0: {"type": "^v"},
                            1: {"type": "^{__CVDisplayLink=}"},
                            2: {
                                "type": "^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                                "type_modifier": "n",
                            },
                            3: {
                                "type": "^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                                "type_modifier": "n",
                            },
                            4: {"type": "Q"},
                            5: {"type": "o^Q"},
                        },
                    }
                }
            }
        },
    ),
    "CVYCbCrMatrixGetStringForIntegerCodePoint": (b"^{__CFString=}i",),
    "CVOpenGLBufferCreate": (
        sel32or64(
            b"i^{__CFAllocator=}LL^{__CFDictionary=}^^{__CVBuffer=}",
            b"i^{__CFAllocator=}QQ^{__CFDictionary=}^^{__CVBuffer=}",
        ),
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {4: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVPixelBufferPoolCreatePixelBufferWithAuxAttributes": (
        b"i^{__CFAllocator=}^{__CVPixelBufferPool=}^{__CFDictionary=}^^{__CVBuffer=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {3: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVOpenGLTextureCacheFlush": (b"v^{__CVOpenGLTextureCache=}Q",),
    "CVDisplayLinkCreateWithActiveCGDisplays": (
        b"i^^{__CVDisplayLink=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {0: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVDisplayLinkGetNominalOutputVideoRefreshPeriod": (
        b"{_CVTime=qii}^{__CVDisplayLink=}",
    ),
    "CVPixelBufferCreateResolvedAttributesDictionary": (
        b"i^{__CFAllocator=}^{__CFArray=}^^{__CFDictionary=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {2: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CVDisplayLinkSetOutputCallback": (
        b"i^{__CVDisplayLink=}^?^v",
        "",
        {
            "arguments": {
                1: {
                    "callable": {
                        "retval": {"type": b"i"},
                        "arguments": {
                            0: {"type": b"^{__CVDisplayLink=}"},
                            1: {
                                "type": b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                                "type_modifier": "n",
                            },
                            2: {
                                "type": b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                                "type_modifier": "N",
                            },
                            3: {"type": b"Q"},
                            4: {"type": b"^Q", "type_modifier": "o"},
                            5: {"type": b"^v"},
                        },
                    }
                }
            }
        },
    ),
    "CVOpenGLTextureGetName": (b"I^{__CVBuffer=}",),
    "CVOpenGLBufferRelease": (b"v^{__CVBuffer=}",),
    "CVOpenGLTextureRetain": (b"^{__CVBuffer=}^{__CVBuffer=}",),
    "CVOpenGLBufferPoolGetAttributes": (b"^{__CFDictionary=}^{__CVOpenGLBufferPool=}",),
    "CVPixelBufferGetWidthOfPlane": (
        sel32or64(b"L^{__CVBuffer=}L", b"Q^{__CVBuffer=}Q"),
    ),
    "CVBufferPropagateAttachments": (b"v^{__CVBuffer=}^{__CVBuffer=}",),
    "CVPixelBufferPoolRetain": (b"^{__CVPixelBufferPool=}^{__CVPixelBufferPool=}",),
    "CVPixelBufferGetHeight": (sel32or64(b"L^{__CVBuffer=}", b"Q^{__CVBuffer=}"),),
    "CVColorPrimariesGetIntegerCodePointForString": (b"i^{__CFString=}",),
    "CVOpenGLBufferGetTypeID": (sel32or64(b"L", b"Q"),),
    "CVDisplayLinkRelease": (b"v^{__CVDisplayLink=}",),
    "CVBufferGetAttachment": (
        b"@^{__CVBuffer=}^{__CFString=}^I",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "CVDisplayLinkStop": (b"i^{__CVDisplayLink=}",),
    "CVPixelFormatDescriptionCreateWithPixelFormatType": (
        sel32or64(
            b"^{__CFDictionary=}^{__CFAllocator=}L",
            b"^{__CFDictionary=}^{__CFAllocator=}I",
        ),
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CVMetalTextureGetCleanTexCoords": (
        b"v^{__CVBuffer=}^f^f^f^f",
        "",
        {
            "arguments": {
                1: {"c_array_of_fixed_length": 2, "type_modifier": "o"},
                2: {"c_array_of_fixed_length": 2, "type_modifier": "o"},
                3: {"c_array_of_fixed_length": 2, "type_modifier": "o"},
                4: {"c_array_of_fixed_length": 2, "type_modifier": "o"},
            }
        },
    ),
    "CVPixelBufferGetIOSurface": (b"^{__IOSurface=}^{__CVBuffer=}",),
    "CVOpenGLTextureCacheCreateTextureFromImage": (
        b"i^{__CFAllocator=}^{__CVOpenGLTextureCache=}^{__CVBuffer=}^{__CFDictionary=}^^{__CVBuffer=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CVDisplayLinkCreateWithCGDisplays": (
        sel32or64(b"i^Il^^{__CVDisplayLink=}", b"i^Iq^^{__CVDisplayLink=}"),
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                0: {"c_array_length_in_arg": 1, "type_modifier": "n"},
                2: {"already_cfretained": True, "type_modifier": "o"},
            },
        },
    ),
    "CVPixelBufferPoolGetPixelBufferAttributes": (
        b"^{__CFDictionary=}^{__CVPixelBufferPool=}",
    ),
    "CVOpenGLTextureGetTypeID": (sel32or64(b"L", b"Q"),),
    "CVImageBufferIsFlipped": (b"Z^{__CVBuffer=}",),
    "CVMetalTextureGetTexture": (b"@^{__CVBuffer=}",),
    "CVPixelBufferIsPlanar": (b"Z^{__CVBuffer=}",),
    "CVBufferRemoveAllAttachments": (b"v^{__CVBuffer=}",),
    "CVTransferFunctionGetIntegerCodePointForString": (b"i^{__CFString=}",),
    "CVPixelBufferCreateWithBytes": (
        sel32or64(
            b"i^{__CFAllocator=}LLL^vL^?^v^{__CFDictionary=}^^{__CVBuffer=}",
            b"i^{__CFAllocator=}QQI^vQ^?^v^{__CFDictionary=}^^{__CVBuffer=}",
        ),
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                6: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"^v"}},
                    }
                }
            },
        },
    ),
    "CVMetalTextureGetTypeID": (b"Q",),
    "CVOpenGLBufferPoolRetain": (b"^{__CVOpenGLBufferPool=}^{__CVOpenGLBufferPool=}",),
    "CVPixelBufferCreateWithPlanarBytes": (
        sel32or64(
            b"i^{__CFAllocator=}LLL^vLL^^v^L^L^L^?^v^{__CFDictionary=}^^{__CVBuffer=}",
            b"i^{__CFAllocator=}QQI^vQQ^^v^Q^Q^Q^?^v^{__CFDictionary=}^^{__CVBuffer=}",
        ),
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                11: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"^v"},
                            2: {"type": b"Q"},
                            3: {"type": b"Q"},
                            4: {"type": b"^^v"},
                        },
                    }
                }
            },
        },
    ),
    "CVImageBufferGetCleanRect": (
        sel32or64(
            b"{CGRect={CGPoint=ff}{CGSize=ff}}^{__CVBuffer=}",
            b"{CGRect={CGPoint=dd}{CGSize=dd}}^{__CVBuffer=}",
        ),
    ),
    "CVImageBufferCreateColorSpaceFromAttachments": (
        b"^{CGColorSpace=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CVPixelBufferGetBytesPerRowOfPlane": (
        sel32or64(b"L^{__CVBuffer=}L", b"Q^{__CVBuffer=}Q"),
    ),
    "CVColorPrimariesGetStringForIntegerCodePoint": (b"^{__CFString=}i",),
    "CVDisplayLinkGetTypeID": (sel32or64(b"L", b"Q"),),
    "CVImageBufferGetDisplaySize": (
        sel32or64(b"{CGSize=ff}^{__CVBuffer=}", b"{CGSize=dd}^{__CVBuffer=}"),
    ),
    "CVPixelBufferGetDataSize": (sel32or64(b"L^{__CVBuffer=}", b"Q^{__CVBuffer=}"),),
    "CVOpenGLBufferPoolGetOpenGLBufferAttributes": (
        b"^{__CFDictionary=}^{__CVOpenGLBufferPool=}",
    ),
    "CVOpenGLBufferAttach": (b"i^{__CVBuffer=}^{_CGLContextObject=}Iii",),
    "CVPixelBufferGetBaseAddressOfPlane": (
        sel32or64(b"^v^{__CVBuffer=}L", b"^v^{__CVBuffer=}Q"),
        "",
        {"retval": {"c_array_of_variable_length": True}},
    ),
    "CVDisplayLinkIsRunning": (b"Z^{__CVDisplayLink=}",),
    "CVPixelBufferGetPixelFormatType": (
        sel32or64(b"L^{__CVBuffer=}", b"I^{__CVBuffer=}"),
    ),
    "CVBufferRemoveAttachment": (b"v^{__CVBuffer=}^{__CFString=}",),
    "CVOpenGLBufferGetAttributes": (b"^{__CFDictionary=}^{__CVBuffer=}",),
    "CVDisplayLinkGetOutputVideoLatency": (b"{_CVTime=qii}^{__CVDisplayLink=}",),
    "CVPixelBufferGetBytesPerRow": (sel32or64(b"L^{__CVBuffer=}", b"Q^{__CVBuffer=}"),),
    "CVMetalTextureCacheCreate": (
        b"i^{__CFAllocator=}^{__CFDictionary=}@^{__CFDictionary=}^^{__CVMetalTextureCache=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CVPixelBufferGetExtendedPixels": (
        sel32or64(b"v^{__CVBuffer=}^L^L^L^L", b"v^{__CVBuffer=}^Q^Q^Q^Q"),
        "",
        {
            "arguments": {
                1: {"type_modifier": "o"},
                2: {"type_modifier": "o"},
                3: {"type_modifier": "o"},
                4: {"type_modifier": "o"},
            }
        },
    ),
    "CVTransferFunctionGetStringForIntegerCodePoint": (b"^{__CFString=}i",),
    "CVImageBufferGetColorSpace": (b"^{CGColorSpace=}^{__CVBuffer=}",),
    "CVDisplayLinkGetCurrentCGDisplay": (b"I^{__CVDisplayLink=}",),
    "CVDisplayLinkSetCurrentCGDisplayFromOpenGLContext": (
        b"i^{__CVDisplayLink=}^{_CGLContextObject=}^{_CGLPixelFormatObject=}",
    ),
    "CVPixelBufferRelease": (b"v^{__CVBuffer=}",),
    "CVBufferSetAttachments": (b"v^{__CVBuffer=}^{__CFDictionary=}I",),
    "CVOpenGLTextureGetTarget": (b"I^{__CVBuffer=}",),
    "CVGetHostClockFrequency": (b"d", "", {"variadic": False}),
    "CVGetHostClockMinimumTimeDelta": (b"I", "", {"variadic": False}),
    "CVOpenGLBufferRetain": (b"^{__CVBuffer=}^{__CVBuffer=}",),
    "CVMetalTextureIsFlipped": (b"Z^{__CVBuffer=}",),
    "CVOpenGLTextureCacheCreate": (
        b"i^{__CFAllocator=}^{__CFDictionary=}^{_CGLContextObject=}^{_CGLPixelFormatObject=}^{__CFDictionary=}^^{__CVOpenGLTextureCache=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CVPixelBufferPoolFlush": (b"v^{__CVPixelBufferPool=}Q",),
    "CVOpenGLTextureGetCleanTexCoords": (b"v^{__CVBuffer=}[2f][2f][2f][2f]",),
}
aliases = {
    "COREVIDEO_USE_DERIVED_ENUMS_FOR_CONSTANTS": "COREVIDEO_TRUE",
    "COREVIDEO_SUPPORTS_PREFETCH": "COREVIDEO_TRUE",
    "CVImageBufferRef": "CVBufferRef",
    "COREVIDEO_SUPPORTS_DIRECT3D": "COREVIDEO_FALSE",
    "COREVIDEO_SUPPORTS_GLES_TEX_IMAGE_IOSURFACE": "COREVIDEO_SUPPORTS_IOSURFACE",
    "COREVIDEO_SUPPORTS_OPENGL": "COREVIDEO_TRUE",
    "COREVIDEO_DECLARE_NULLABILITY": "COREVIDEO_TRUE",
    "COREVIDEO_SUPPORTS_IOSURFACE_PREFETCH": "COREVIDEO_FALSE",
    "COREVIDEO_SUPPORTS_IOSURFACE": "COREVIDEO_TRUE",
    "COREVIDEO_SUPPORTS_OPENGLES": "COREVIDEO_FALSE",
    "COREVIDEO_SUPPORTS_COLORSPACE": "COREVIDEO_TRUE",
    "CV_INLINE": "CF_INLINE",
    "CV_RELEASES_ARGUMENT": "CF_RELEASES_ARGUMENT",
    "CV_RETURNS_RETAINED_PARAMETER": "CF_RETURNS_RETAINED",
    "COREVIDEO_SUPPORTS_DISPLAYLINK": "COREVIDEO_TRUE",
    "CV_NULLABLE": "__nullable",
    "COREVIDEO_USE_IOSURFACEREF": "COREVIDEO_FALSE",
    "CV_NONNULL": "__nonnull",
    "kCVReturnError": "kCVReturnFirst",
    "COREVIDEO_SUPPORTS_METAL": "COREVIDEO_TRUE",
}
cftypes = [
    ("CVBufferRef", b"^{__CVBuffer=}", "CVBufferGetTypeID", None),
    ("CVDisplayLinkRef", b"^{__CVDisplayLink=}", "CVDisplayLinkGetTypeID", None),
    ("CVMetalTextureCacheRef", b"^{__CVMetalTextureCache=}", None, None),
    (
        "CVOpenGLBufferPoolRef",
        b"^{__CVOpenGLBufferPool=}",
        "CVOpenGLBufferPoolGetTypeID",
        None,
    ),
    (
        "CVOpenGLTextureCacheRef",
        b"^{__CVOpenGLTextureCache=}",
        "CVOpenGLTextureCacheGetTypeID",
        None,
    ),
    (
        "CVPixelBufferPoolRef",
        b"^{__CVPixelBufferPool=}",
        "CVPixelBufferPoolGetTypeID",
        None,
    ),
    ("CVOpenGLBufferRef", b"^{__CVOpenGLBuffer=}", "CVOpenGLBufferGetTypeID", None),
    ("CVPixelBufferRef", b"^{__CVPixelBuffer=}", "CVPixelBufferGetTypeID", None),
    ("CVOpenGLTextureRef", b"^{__CVOpenGLTexture=}", "CVOpenGLTextureGetTypeID", None),
]
expressions = {}

# END OF FILE



"""
Python mapping for the ImageKit framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import Cocoa
import objc
from Quartz.ImageKit import _metadata
from Quartz.ImageKit import _imagekit

objc.addConvenienceForBasicMapping("IKImageBrowserGridGroup", False)
objc.addConvenienceForBasicMapping("IKImageCell", False)
objc.addConvenienceForBasicMapping("IKImageState", False)
objc.addConvenienceForBasicSequence("IKLinkedList", True)

sys.modules["Quartz.ImageKit"] = mod = objc.ObjCLazyModule(
    "Quartz.ImageKit",
    "com.apple.imageKit",
    objc.pathForFramework(
        "/System/Library/Frameworks/Quartz.framework/Frameworks/ImageKit.framework"
    ),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (_imagekit, Cocoa),
)


del sys.modules["Quartz.ImageKit._metadata"]



import math

import Cocoa
import objc
import Quartz

TRANSITION_COUNT = 9


class TransitionSelectorView(Cocoa.NSView):
    _sourceImage = objc.ivar()
    _targetImage = objc.ivar()
    _shadingImage = objc.ivar()
    _blankImage = objc.ivar()
    _maskImage = objc.ivar()
    transitions = objc.ivar()  # Array

    base = objc.ivar(type=objc._C_DBL)

    thumbnailWidth = objc.ivar(type=objc._C_FLT)
    thumbnailHeight = objc.ivar(type=objc._C_FLT)
    thumbnailGap = objc.ivar(type=objc._C_FLT)

    def awakeFromNib(self):
        self.thumbnailWidth = 340.0
        self.thumbnailHeight = 240.0
        self.thumbnailGap = 20.0

        url = Cocoa.NSURL.fileURLWithPath_(
            Cocoa.NSBundle.mainBundle().pathForResource_ofType_("Rose", "jpg")
        )
        self.setSourceImage_(Quartz.CIImage.imageWithContentsOfURL_(url))

        url = Cocoa.NSURL.fileURLWithPath_(
            Cocoa.NSBundle.mainBundle().pathForResource_ofType_("Frog", "jpg")
        )
        self.setTargetImage_(Quartz.CIImage.imageWithContentsOfURL_(url))

        timer = Cocoa.NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(
            1.0 / 30.0, self, "timerFired:", None, True
        )

        self.base = Cocoa.NSDate.pyobjc_classMethods.timeIntervalSinceReferenceDate()
        Cocoa.NSRunLoop.currentRunLoop().addTimer_forMode_(
            timer, Cocoa.NSDefaultRunLoopMode
        )
        Cocoa.NSRunLoop.currentRunLoop().addTimer_forMode_(
            timer, Cocoa.NSEventTrackingRunLoopMode
        )

    def setSourceImage_(self, source):
        self._sourceImage = source

    def setTargetImage_(self, target):
        self._targetImage = target

    def shadingImage(self):
        if self._shadingImage is None:
            url = Cocoa.NSURL.fileURLWithPath_(
                Cocoa.NSBundle.mainBundle().pathForResource_ofType_("Shading", "tiff")
            )
            self._shadingImage = Quartz.CIImage.alloc().initWithContentsOfURL_(url)

        return self._shadingImage

    def blankImage(self):
        if self._blankImage is None:
            url = Cocoa.NSURL.fileURLWithPath_(
                Cocoa.NSBundle.mainBundle().pathForResource_ofType_("Blank", "jpg")
            )
            self._blankImage = Quartz.CIImage.alloc().initWithContentsOfURL_(url)

        return self._blankImage

    def maskImage(self):
        if self._maskImage is None:
            url = Cocoa.NSURL.fileURLWithPath_(
                Cocoa.NSBundle.mainBundle().pathForResource_ofType_("Mask", "jpg")
            )
            self._maskImage = Quartz.CIImage.alloc().initWithContentsOfURL_(url)

        return self._maskImage

    def timerFired_(self, sender):
        self.setNeedsDisplay_(True)

    def imageForTransition_atTime_(self, transitionNumber, t):
        transition = self.transitions[transitionNumber]

        if t % 2.0 < 1.0:
            transition.setValue_forKey_(self._sourceImage, "inputImage")
            transition.setValue_forKey_(self._targetImage, "inputTargetImage")
        else:
            transition.setValue_forKey_(self._targetImage, "inputImage")
            transition.setValue_forKey_(self._sourceImage, "inputTargetImage")

        transition._.inputTime = t % 1.0

        crop = Quartz.CIFilter.filterWithName_("CICrop")
        crop._.inputImage = transition._.outputImage
        crop._.inputRectangle = Quartz.CIVector.vectorWithX_Y_Z_W_(
            0, 0, self.thumbnailWidth, self.thumbnailHeight
        )
        return crop._.outputImage

    def drawRect_(self, rectangle):
        thumbFrame = Quartz.CGRectMake(0, 0, self.thumbnailWidth, self.thumbnailHeight)
        t = 0.4 * (
            Cocoa.NSDate.pyobjc_classMethods.timeIntervalSinceReferenceDate()
            - self.base
        )

        context = Cocoa.NSGraphicsContext.currentContext().CIContext()

        if self.transitions is None:
            self.transitions = [None] * TRANSITION_COUNT
            self.setupTransitions()

        w = int(math.ceil(math.sqrt(TRANSITION_COUNT)))
        origin = Quartz.CGPoint()

        for i in range(TRANSITION_COUNT):
            origin.x = (i % w) * (self.thumbnailWidth + self.thumbnailGap)
            origin.y = (i / w) * (self.thumbnailHeight + self.thumbnailGap)

            if context is not None:
                context.drawImage_atPoint_fromRect_(
                    self.imageForTransition_atTime_(i, t + 0.1 * i), origin, thumbFrame
                )

    def setupTransitions(self):
        w = self.thumbnailWidth
        h = self.thumbnailHeight

        extent = Quartz.CIVector.vectorWithX_Y_Z_W_(0, 0, w, h)

        self.transitions[0] = Quartz.CIFilter.filterWithName_("CISwipeTransition")
        self.transitions[0]._.inputExtent = extent
        self.transitions[
            0
        ]._.inputColor = Quartz.CIColor.colorWithRed_green_blue_alpha_(0, 0, 0, 0)
        self.transitions[0]._.inputAngle = 0.3 * math.pi
        self.transitions[0]._.inputWidth = 80.0
        self.transitions[0]._.inputOpacity = 0.0

        self.transitions[1] = Quartz.CIFilter.filterWithName_("CIDissolveTransition")

        self.transitions[2] = Quartz.CIFilter.filterWithName_(
            "CISwipeTransition"
        )  # dupe
        self.transitions[2]._.inputExtent = extent
        self.transitions[
            2
        ]._.inputColor = Quartz.CIColor.colorWithRed_green_blue_alpha_(0, 0, 0, 0)
        self.transitions[2]._.inputAngle = math.pi
        self.transitions[2]._.inputWidth = 2.0
        self.transitions[2]._.inputOpacity = 0.2

        self.transitions[3] = Quartz.CIFilter.filterWithName_("CIModTransition")
        self.transitions[3]._.inputCenter = Quartz.CIVector.vectorWithX_Y_(
            0.5 * w, 0.5 * h
        )
        self.transitions[3]._.inputAngle = math.pi * 0.1
        self.transitions[3]._.inputRadius = 30.0
        self.transitions[3]._.inputCompression = 10.0

        self.transitions[4] = Quartz.CIFilter.filterWithName_("CIFlashTransition")
        self.transitions[4]._.inputExtent = extent
        self.transitions[4]._.inputCenter = Quartz.CIVector.vectorWithX_Y_(
            0.3 * w, 0.7 * h
        )
        self.transitions[
            4
        ]._.inputColor = Quartz.CIColor.colorWithRed_green_blue_alpha_(1, 0.8, 0.6, 1)
        self.transitions[4]._.inputMaxStriationRadius = 2.5
        self.transitions[4]._.inputStriationStrength = 0.5
        self.transitions[4]._.inputStriationContrast = 1.37
        self.transitions[4]._.inputFadeThreshold = 0.85

        self.transitions[5] = Quartz.CIFilter.filterWithName_(
            "CIDisintegrateWithMaskTransition"
        )
        self.transitions[5]._.inputMaskImage = self.maskImage()
        self.transitions[5]._.inputShadowRadius = 10.0
        self.transitions[5]._.inputShadowDensity = 0.7
        self.transitions[5]._.inputShadowOffset = Quartz.CIVector.vectorWithX_Y_(
            0.0, -0.05 * h
        )

        self.transitions[6] = Quartz.CIFilter.filterWithName_("CIRippleTransition")
        self.transitions[6]._.inputExtent = extent
        self.transitions[6]._.inputShadingImage = self.shadingImage()
        self.transitions[6]._.inputCenter = Quartz.CIVector.vectorWithX_Y_(
            0.5 * w, 0.5 * h
        )
        self.transitions[6]._.inputWidth = 80.0
        self.transitions[6]._.inputScale = 30.0

        self.transitions[7] = Quartz.CIFilter.filterWithName_("CICopyMachineTransition")
        self.transitions[7]._.inputExtent = extent
        self.transitions[
            7
        ]._.inputColor = Quartz.CIColor.colorWithRed_green_blue_alpha_(0.6, 1, 0.8, 1)
        self.transitions[7]._.inputAngle = 0
        self.transitions[7]._.inputWidth = 40
        self.transitions[7]._.inputOpacity = 1.0

        self.transitions[8] = Quartz.CIFilter.filterWithName_("CIPageCurlTransition")
        self.transitions[8]._.inputExtent = extent
        self.transitions[8]._.inputShadingImage = self.shadingImage()
        self.transitions[8]._.inputBacksideImage = self.blankImage()
        self.transitions[8]._.inputAngle = -0.2 * math.pi
        self.transitions[8]._.inputRadius = 70.0
